#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import json
import re
import unicodedata

from math import log10

class WordExtractionTFIDF(object):
    def __init__(self):
        self.thd = 0.06
        self.idfDICT, self.docCount = self._getIdfDict("idf.json")

    def __str__(self):
        return "Articut API KeywordExtraction"

    def _getIdfDict(self, fn):
        import os
        fn = "{}/data/{}".format(os.path.dirname(os.path.abspath(__file__)),fn)
        try:
            aidf = json.load(open(fn, "r", encoding=("UTF-8")))
            # idfDICT = {"":[fn id list], w:[ids, ids, ...], ...}
            return aidf, len(aidf[""])
        except Exception as e:
            print("idf dict import error.")
            print(str(e))
            return None

    def eval(self, wct, wlst, dct, idfd):
        # wlst = {w:ct, w:ct, ...}
        # idfd= {w:[ids, ids, ...], ...}
        d = {}
        for w in wlst:
            if "" == w:
                continue
            if w not in idfd:       # 未知詞彙 wct = 1
                sdt = 1
            else:
                sdt = len(idfd[w])  # 已知詞彙
            # tf, idf, tfidf 計算
            tf = wlst[w]/float(wct)
            idf = log10(float(dct)/sdt)
            tfidf = tf*idf
            # d[w] = (tf, idf, tfidf)
            d[w] = tfidf
        return d

    def getTfList(self, wLIST):
        d = {}
        wct = 0
        for k in wLIST:
            # Ignore ''
            if "" == k:
                continue
            # Ignore punctuation
            if 1 == len(k) and unicodedata.category(k).startswith("P"):
                continue
            if k not in d:
                d[k] = 1
                wct += 1
            else:
                d[k] += 1
                wct += 1
        return d, wct

    def sortByTfidf(self, evlDICT):
        # evldict = _evld = {w:(tf,idf,tfidf)}
        lst = list(evlDICT.items())
        # lst = [ (u"\u4e00", 0.02395209580838323), ... ]
        lst = sorted(lst, key=lambda x: x[1], reverse=True)
        return lst

    def extractKeyword(self, inputSTR, topK, withWeight, allowPOS):
        # self.thd = topK/100.0                   # 取結果的前 topK% 個詞
        self.thd = topK                           # 取結果的前 topK 個詞
        if self.thd < 0: self.thd = 20

        # get word list
        wordLIST = inputSTR.split("/")            # wordLIST = ["沒有", "人", ...]

        # get tfDICT and get wct of the inputSTR
        tfDICT, wct = self.getTfList(wordLIST)    # tfDICT = {"沒有":1, "命運":        2, ...}

        # get tfidf dict
        evalDICT = self.eval(wct, tfDICT, self.docCount, self.idfDICT)    # evalDict = {w:tfidf, ...}

        # sort by tfidf
        rst = self.sortByTfidf(evalDICT)

        # filter by threshold
        # keyWordLIST = rst[:round(self.thd*len(rst))]  # 搭配 self.thd = topK/100.0
        keyWordLIST = rst[:self.thd]                    # 搭配 self.thd = topK

        # return key word list
        if withWeight is False:
            newLIST = []
            for k in keyWordLIST:
                newLIST.append(k[0])
            return newLIST
        else:
            return keyWordLIST

class WordExtractionTextRank(object):
    def __init__(self):
        self.dWeight = 0.85
        self.initScoreINT = 1
        self.winSizeINT = 5
        self.stopWordTagsLIST = ["<QUANTIFIER>[^<]*?</QUANTIFIER>",
                                 "<MODAL>[^<]*?</MODAL>",
                                 "<AUX>[^<]*?</AUX>",
                                 "<ASPECT>[^<]*?</ASPECT>",
                                 "<FUNC_.*?>[^<]*?</FUNC_.*?>",
                                 "<CLAUSE_.*?>[^<]*?</CLAUSE_.*?>",
                                 "<RANGE_.*?>[^<]*?</RANGE_.*?>"]
        self.stopWordPat = re.compile("|".join(self.stopWordTagsLIST))
        self.stripPat = re.compile("(?<=>).*?(?=<)")

    def _getScore(self, scoreLIST, outLIST, linksLIST, refID):
        i = 0
        score = 0
        while i < len(scoreLIST):
            if i != refID:
                score += 1-self.dWeight + self.dWeight*(scoreLIST[i] * outLIST[i] / linksLIST[i])
            i += 1
        return score

    def itrRanking(self, wordRefLIST, matrixLIST, iterTimesINT):
        # wordRefLIST = ['命運', '自己', '手', '人', '決定', '你', '命運']
        # matrixLIST = [[0, 1, 1, 0, ...], [1, 0, 2, 2, ...], ... ]
        # iterTimesINT = 5
        linksLIST = []
        for outLIST in matrixLIST:
            ct = 0
            for i in outLIST:
                ct += i
            linksLIST.append(ct)

        scoreLIST = [self.initScoreINT]*len(wordRefLIST)
        t = 0
        while t < iterTimesINT:
            tmpLIST = []
            for i in range(len(matrixLIST)):
                score = self._getScore(scoreLIST, matrixLIST[i], linksLIST, i)
                tmpLIST.append(score)
            scoreLIST = tmpLIST[:]
            t += 1

        return scoreLIST

    def mappingWords(self, scoreLIST, wordRefLIST):
        # wordRefLIST = ['命運', '自己', '手', '人', '決定', '你', '命運']
        # scoreLIST = [1.1575125520833334, 2.438575, 1.9919230208333336, 1.4989894270833335, ...]
        sortLIST = []
        for i in range(len(scoreLIST)):
            sortLIST.append([wordRefLIST[i], scoreLIST[i]])
        sortLIST = sorted(sortLIST, key=lambda x: x[1])
        sortLIST.reverse()
        return sortLIST

    def rankWords(self, wordLIST, iterTimesINT):
        # wordLIST = ['命運', '自己', '手', '人', '決定', '你', '命運']
        wordRefLIST = []
        for w in wordLIST:
            if w not in wordRefLIST:
                wordRefLIST.append(w)
        # wordRefLIST = ['命運', '自己', '決定', '手', '你', '人']
        matrixLIST = [[0]*len(wordRefLIST) for item in range(len(wordRefLIST))]
        # matrixLIST = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0], [0, 0, 0, 0] ]
        # matrixLIST = [['命運', '自己', '決定', '手', '你', '人'],    for（命運）
        #               ['命運', '自己', '決定', '手', '你', '人'],    for（自己）
        #               ['命運', '自己', '決定', '手', '你', '人'],    for（決定）
        #               ['命運', '自己', '決定', '手', '你', '人'],    for（手）
        #                ...... ]

        for i in range(len(wordLIST)):
            startOffsetINT = i-self.winSizeINT
            endOffsetINT = i+self.winSizeINT+1
            if i-self.winSizeINT < 0:
                startOffsetINT = 0
            # print(wordLIST[startOffsetINT:i], wordLIST[i], wordLIST[i+1:endOffsetINT])

            for w in wordLIST[startOffsetINT:i] + wordLIST[i+1:endOffsetINT]:
                matrixLIST[wordRefLIST.index(wordLIST[i])][wordRefLIST.index(w)] += 1

        scoreLIST = self.itrRanking(wordRefLIST, matrixLIST, iterTimesINT)
        wordRankLIST = self.mappingWords(scoreLIST, wordRefLIST)
        return wordRankLIST

    def extractWords(self, resultLIST):
        resultSTR = "".join([x for x in resultLIST if len(x)>1])
        wordLIST = []
        stopwordLIST = [x.group() for x in self.stopWordPat.finditer(resultSTR)]
        for sw in stopwordLIST:
            resultSTR = resultSTR.replace(sw, "")
        wordLIST = [x.group() for x in self.stripPat.finditer(resultSTR) if len(x.group())>0]
        return wordLIST

    def extractKeyword(self, resultDICT, iterTimesINT=20, withWeight=False, allowPOS=('ns', 'n', 'vn', 'v')):
        resultLIST = resultDICT['result_pos']
        # resultLIST =
        # [ '<articut_tag>沒有人可以決定你的命運</articut_tag>',
        #   '，',
        #   '<articut_tag>命運在自己的手上</articut_tag>',
        #   '。' ]
        wordLIST = self.extractWords(resultLIST)
        # wordLIST = ['命運', '自己', '決定', '手', '你', '人']
        wordRankLIST = self.rankWords(wordLIST, iterTimesINT)
        # wordRankLIST = [['命運', 5.591625666787958],
        #                 ['自己', 3.4637188376903927],
        #                 ['你', 3.4637188376903927],
        #                 ['決定', 3.4637188376903927],
        #                 ['手', 2.959546855830475],
        #                 ['人', 2.9595468558304745]]
        if iterTimesINT < 0:
            iterTimesINT = 20
        if withWeight is False:
            tmpLIST = []
            for w in wordRankLIST:
                tmpLIST.append(w[0])
            return tmpLIST[:iterTimesINT]
        else:
            return wordRankLIST[:iterTimesINT]

class AnalyseManager(object):
    def __init__(self):
        self.tfidfOBJ = WordExtractionTFIDF()
        self.txtrankOBJ = WordExtractionTextRank()
        self.POSDICT = {'n':'', 'v':'', 'a':'', 'p':''}

    def convertPOS(self, POSTOUPLE):
        return POSTOUPLE

    def TFIDF(self, idf_path=None):
        if idf_path != None:
            self.tfidfOBJ._getIdfDict(idf_path)

    def extractTags(self, resultDICT, topK=20, withWeight=False, allowPOS=('ns', 'n', 'vn', 'v')):
        if "result_segmentation" in resultDICT:
            pass
        else:
            return None
        try:
            allowPOS = self.convertPOS(allowPOS)
            result = self.tfidfOBJ.extractKeyword(resultDICT["result_segmentation"], topK, withWeight, allowPOS)
            return result
        except Exception as e:
            print(str(e))
            return None

    def extract_tags(self, resultDICT, topK=20, withWeight=False, allowPOS=('ns', 'n', 'vn', 'v')):
        # Alias of extractTags() for developers from older CWS solutions.
        return self.extractTags(resultDICT, topK, withWeight, allowPOS=('ns', 'n', 'vn', 'v'))

    def textrank(self, resultDICT, topK=20, withWeight=False, allowPOS=('ns', 'n', 'vn', 'v')):
        # Key word extraction and rank by the TextRank algorithm
        # Alias of jieba textrank()
        # topK = iterative times
        if "result_segmentation" in resultDICT and "result_pos" in resultDICT:
            pass
        else:
            return None
        try:
            allowPOS = self.convertPOS(allowPOS)
            return self.txtrankOBJ.extractKeyword(resultDICT, topK, withWeight, allowPOS)
        except Exception as e:
            print(str(e))
            return None



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

from time import sleep
import os
import re

try:
    import rapidjson as json
except:
    import json

try:
    import requests
except Exception as e:
    import platform
    if "windows" in platform.system().lower():
        print("""
        Module, \"requests\" is not installed.
        If you don't know how to install "requests" under Windows,
        please use our request_installer.py in "Windows_Module_Installer" folder.
        """)
    else:
        print(e)

try:
    from Toolkit.analyse import AnalyseManager
    from Toolkit.localRE import TaiwanAddressAnalizer
    from Toolkit.toolkits import *
    from Toolkit.NER import GenericNER
except: #供外部載入時使用。
    from .Toolkit.analyse import AnalyseManager
    from .Toolkit.localRE import TaiwanAddressAnalizer
    from .Toolkit.toolkits import *
    from .Toolkit.NER import GenericNER

#try:
    #from Toolkit.graphQL import GraphQL
#except:
    #print("No module named 'graphene'")
    #print("Articut-graphQL requires 'graphene' module.")
    #print("Please use pip/conda install graphene-python to install the module and reload ArticutAPI.")

class Articut:
    def __init__(self, username="", apikey="", version="latest", level="lv2", url="https://api.droidtown.co"):
        '''
        username = ""    # 你註冊時的 email。若留空，則會使用每小時更新 2000 字的公用帳號。
        apikey = ""      # 您完成付費後取得的 apikey 值。若留空，則會使用每小時更新 2000 字的公用帳號。
        '''
        self.MAX_LEN = 5000
        self.RETRY_COUNT = 1
        self.RETRY_DELAY = 10 # 10 sec

        try:
            with open("./account.info", "r") as f:
                userDICT = json.loads(f.read())
            self.username = userDICT["email"]
            self.apikey = userDICT["apikey"]
        except:
            self.username = username
            self.apikey = apikey

        self.url = url

        self.version = version
        self.level = level

        self.userDefinedDictFILE = None
        self.openDataPlaceAccessBOOL=False
        self.chemicalBOOL=True
        self.fileSizeLimit = 1024 * 1024 * 10    # 10 MB

        # Toolkit
        self.analyse = AnalyseManager()
        self.localRE = TaiwanAddressAnalizer(locale="TW")
        self.LawsToolkit = LawsToolkit()
        self.NER = GenericNER()
        self.POS = ArticutPOS()

        #try:
            #self.graphQL = GraphQL()
        #except:
            #pass

    def __str__(self):
        return "Articut API"

    def parse(self, inputSTR, level="", userDefinedDictFILE=None, chemicalBOOL=True, openDataPlaceAccessBOOL=False, wikiDataBOOL=False, indexWithPOS=False, timeRef=None, pinyin="BOPOMOFO", autoBreakBOOL=True):
        if level not in ("lv1", "lv2", "lv3"):
            level = self.level

        self.openDataPlaceAccessBOOL=openDataPlaceAccessBOOL
        self.wikiDataBOOL=wikiDataBOOL
        self.chemicalBOOL=chemicalBOOL
        url = "{}/Articut/API/".format(self.url)
        if level in ("lv1", "lv2"):
            payload = {"username": self.username,                     #String Type：使用者帳號 email
                       "api_key": self.apikey,                        #String Type：使用者 api key。若未提供，預設使用每小時更新 2000 字的公用額度。
                       "version": self.version,                       #String Type：指定斷詞引擎版本號。預設為最新版 "latest"
                       "level": level,                                #String Type：指定為 lv1 極致斷詞 (斷得較細) 或 lv2 詞組斷詞 (斷得較粗)。
                       "chemical": self.chemicalBOOL,                 #Bool Type：為 True 或 False，表示是否允許 Articut 讀取 Chemical 偵測化學類名稱。
                       "opendata_place":self.openDataPlaceAccessBOOL, #Bool Type：為 True 或 False，表示是否允許 Articut 讀取 OpenData 中的地點名稱。
                       "wikidata": self.wikiDataBOOL}                 #Bool Type：為 True 或 False，表示是否允許 Articut 讀取 WikiData 中的條目名稱。
        else:
            payload = {"username": self.username,                     #String Type：使用者帳號 email
                       "api_key": self.apikey,                        #String Type：使用者 api key。若未提供，預設使用每小時更新 2000 字的公用額度。
                       "version": self.version,                       #String Type：指定斷詞引擎版本號。預設為最新版 "latest"
                       "level": level,                                #String Type：指定為 lv3 語意斷詞。
                       "chemical": self.chemicalBOOL,                 #Bool Type：為 True 或 False，表示是否允許 Articut 讀取 Chemical 偵測化學類名稱。
                       "opendata_place":self.openDataPlaceAccessBOOL, #Bool Type：為 True 或 False，表示是否允許 Articut 讀取 OpenData 中的地點名稱。
                       "wikidata": self.wikiDataBOOL,                 #Bool Type：為 True 或 False，表示是否允許 Articut 讀取 WikiData 中的條目名稱。
                       "index_with_pos":False,
                       "pinyin":pinyin
            }
            if timeRef:
                payload["time_ref"] = str(timeRef)


        if userDefinedDictFILE:
            try:
                if os.path.getsize(userDefinedDictFILE) <= self.fileSizeLimit:
                    userDefinedFile = json.load(open(userDefinedDictFILE, "r", encoding="utf8"))
                    if type(userDefinedFile) == dict:
                        payload["user_defined_dict_file"] = userDefinedFile
                    else:
                        print("User Defined File must be dict type.")
                        return {"status": False, "msg": "UserDefinedDICT Parsing ERROR. Please check your the format and encoding."}
                else:
                    print("Maximum file size limit is 10 MB.")
            except Exception as e:
                print("User Defined File Loading Error.")
                print(str(e))
                return {"status": False, "msg": "UserDefinedDICT Parsing ERROR. Please check your the format and encoding."}

        if autoBreakBOOL:
            inputLIST = self._getInputLIST(inputSTR)
        else:
            inputLIST = [inputSTR]

        resultDICT = {}
        count = 0
        for x in inputLIST:
            payload["input_str"] = x    #String Type：要做斷詞處理的中文句子。
            retry_count = 0
            while True:
                try:
                    result = requests.post(url, json=payload)
                    if result.status_code == 200:
                        result = result.json()
                        if not result["status"]:
                            return result

                        if resultDICT:
                            resultDICT["exec_time"] += result["exec_time"]
                            resultDICT["word_count_balance"] = result["word_count_balance"]
                            if level in ("lv1", "lv2"):
                                resultDICT["result_obj"].extend(result["result_obj"])
                                resultDICT["result_pos"].extend(result["result_pos"])
                                resultDICT["result_segmentation"] += "/{}".format(result["result_segmentation"])
                            else:
                                resultDICT["input"].extend([[i[0] + count, i[1] + count] for i in result["input"]])
                                resultDICT["entity"].extend(result["entity"])
                                resultDICT["event"].extend(result["event"])
                                resultDICT["person"].extend(result["person"])
                                resultDICT["site"].extend(result["site"])
                                resultDICT["time"].extend(result["time"])
                                resultDICT["user_defined"].extend(result["user_defined"])
                                resultDICT["utterance"].extend(result["utterance"])
                                resultDICT["number"] = {**resultDICT["number"], **result["number"]}
                                resultDICT["unit"] = {**resultDICT["unit"], **result["unit"]}
                        else:
                            resultDICT = result
                        count += len(x)
                    else:
                        return result

                    # 成功取得結果跳出 while 迴圈
                    break

                except Exception as e:
                    print("[ERROR] {} {}\n=> {}".format(retry_count, str(e), payload["input_str"]))
                    # 最多嘗試 RETRY_COUNT 次
                    if retry_count < self.RETRY_COUNT:
                        retry_count += 1
                        sleep(self.RETRY_DELAY)
                    else:
                        return {"status": False, "msg": "Connection timeout."}

        return resultDICT

    def _getInputLIST(self, inputSTR):
        '''
        取得長度不大於 MAX_LEN 的 input 列表
        '''
        BREAK_LIST = ["。", "？", "！", "?", "!", "\n"]

        inputLIST = []
        while True:
            if len(inputSTR) > self.MAX_LEN:
                tempSTR = inputSTR[:self.MAX_LEN]
                index = 0
                for x in BREAK_LIST:
                    lastIndex = tempSTR.rfind(x) + 1
                    if lastIndex > index:
                        index = lastIndex
                if index == 0:
                    index = self.MAX_LEN
                inputLIST.append(inputSTR[:index])
                inputSTR = inputSTR[index:]
            else:
                inputLIST.append(inputSTR)
                break

        return inputLIST

    def versions(self):
        url = "{}/Articut/Versions/".format(self.url)
        payload = {"username":  self.username,
                   "api_key":   self.apikey}
        result = requests.post(url, data=payload)
        if result.status_code == 200:
            result = result.json()
            result["product"] = "{}/product/".format(self.url)
            result["document"] = "{}/document/".format(self.url)
        return result

    ##############################################################################
    #                                 Toolkits                                   #
    ##############################################################################
    def getAddTWLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中含有 (KNOWLEDGE_addTW) 標籤的字串。
        該字串為一台灣地址。
        '''
        return self.POS.getAddTWLIST(parseResultDICT, indexWithPOS)

    def getChemicalLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的化學類詞 (KNOWLEDGE_chemical)。
        每個句子內的化學類詞為一個 list。
        '''
        return self.POS.getChemicalLIST(parseResultDICT, indexWithPOS)

    def getColorLIST(self, resultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中含有 (MODIFIER_color) 標籤的字串。
        該字串為一顏色表述字串。
        '''
        return self.POS.getColorLIST(resultDICT, indexWithPOS)

    def getContentWordLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的實詞 (content word)。
        每個句子內的實詞為一個 list。
        '''
        return self.POS.getContentWordLIST(parseResultDICT, indexWithPOS)

    def getCurrencyLIST(self, parseResultDICT, indexWithPOS=True, greedyBOOL=False):
        '''
        取出斷詞結果中的貨幣金額 (KNOWLEDGE_currency) 標籤的字串。
        每個句子內的「貨幣金額」，將列為一個 list。
        若 greedy = True，則以下格式會加到回傳 list
            貨幣名稱 + 數字 (包含「'」與「,」符號)
            新台幣 100
            美金9.99
            歐元 1,999'99
        '''
        return self.POS.getCurrencyLIST(parseResultDICT, indexWithPOS, greedyBOOL)

    def getLocationStemLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的地理位置 (location)。此處指的是地理位置標記的行政區地名詞彙，例如「台北」、「桃園」、「墨西哥」。
        每個句子內的地理位置列為一個 list。
        '''
        return self.POS.getLocationStemLIST(parseResultDICT, indexWithPOS)

    def getNounStemLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的名詞 (noun)。此處指的是 ENTITY_noun、ENTITY_nouny、ENTITY_nounHead 或 ENTITY_oov 標記的名詞詞彙。
        每個句子內的名詞為一個 list。
        '''
        return self.POS.getNounStemLIST(parseResultDICT, indexWithPOS)

    def getOpenDataPlaceLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的景點 (KNOWLEDGE_place) 標籤的字串。此處指的是景點 (KNOWLEDGE_place)標記的非行政地點名稱詞彙，例如「鹿港老街」、「宜蘭運動公園」。
        每個句子內的景點為一個 list.
        '''
        return self.POS.getOpenDataPlaceLIST(parseResultDICT, indexWithPOS)

    def getPersonLIST(self, parseResultDICT, includePronounBOOL=True, indexWithPOS=True):
        '''
        取出斷詞結果中的人名 (Person)
        若 includePronounBOOL 為 True，則連代名詞 (Pronoun) 一併回傳；若為 False，則只回傳人名。
        回傳結果為一個 list。
        '''
        return self.POS.getPersonLIST(parseResultDICT, includePronounBOOL, indexWithPOS)

    def getQuestionLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中含有 (CLAUSE_Q) 標籤的句子。
        此處指的是
            <CLAUSE_AnotAQ>: A-not-A 問句
            <CLAUSE_YesNoQ>: 是非問句
            <CLAUSE_WhoQ">: 「誰」問句
            <CLAUSE_WhatQ>: 「物」問句
            <CLAUSE_WhereQ>: 「何地」問句
            <CLAUSE_WhenQ>: 「何時」問句
            <CLAUSE_HowQ>: 「程度/過程」問句
            <CLAUSE_WhyQ>: 「原因」問句
        每個句子內若有 <CLAUSE_Q> 標籤，整個句子將會存進 list。
        '''
        return self.POS.getQuestionLIST(parseResultDICT, indexWithPOS)

    def getTimeLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的時間 (time)。
        每個句子內的「時間」詞列為一個 list。
        '''
        return self.POS.getTimeLIST(parseResultDICT, indexWithPOS)

    def getVerbStemLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的動詞 (verb)。此處指的是 ACTION_verb 標記的動詞詞彙。
        每個句子內的動詞為一個 list。
        '''
        return self.POS.getVerbStemLIST(parseResultDICT, indexWithPOS)

    def getWikiDataLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的 WikiData 標記文字。此處指的是 KNOWLEDGE_wikiData 標記的條目名稱。
        每個句子內的條目名稱為一個 list。
        '''
        return self.POS.getWikiDataLIST(parseResultDICT, indexWithPOS)


if __name__ == "__main__":
    from pprint import pprint


    #inputSTR = "你計劃過地球人類補完計劃" #parse() Demo
    #inputSTR = "2018 年 7 月 26 日" #getTimeLIST() Demo
    #inputSTR = "蔡英文總統明日到台北市政府找柯文哲開會討論他的想法，請你安排一下！" #getPersonLIST() Demo
    #inputSTR = "地址：宜蘭縣宜蘭市縣政北七路六段55巷1號2樓" #localRE 工具包 Demo
    inputSTR = "劉克襄在本次活動當中，分享了台北中山北路一日遊路線。他表示當初自己領著柯文哲一同探索了雙連市場與中山捷運站的小吃與商圈，還有商圈內的文創商店與日系雜物店鋪，都令柯文哲留下深刻的印象。劉克襄也認為，雙連市場內的魯肉飯、圓仔湯與切仔麵，還有九條通的日式店家、居酒屋等特色，也能讓人感受到台北舊城區不一樣的魅力。" #Articut-GraphQL Demo
    #inputSTR = "業經前案判決非法持有可發射子彈具殺傷力之槍枝罪"
    #inputSTR = "劉克襄在本次活動當中，分享了台北中山北路一日遊路線。"
    inputSTR = "在常溫下可將銀氧化成氧化銀"
    articut = Articut()

    print("inputSTR:{}\n".format(inputSTR))

    #檢查儲存的結果是否已存在
    resultFilePath = "articutResult.json"
    resultExistBOOL = False
    try:
        with open(resultFilePath, "r", encoding="utf-8") as resultFile:
            resultDICT = json.loads(resultFile.read())
            # inputSTR 去除空白及斜線
            # result_segmentation 去除斜線
            if inputSTR.replace(' ', '').replace("/", "") == resultDICT["result_segmentation"].replace("/", ""):
                resultExistBOOL = True
    except:
        pass

    #取得斷詞結果
    if not resultExistBOOL:
        resultDICT = articut.parse(inputSTR, level="lv2", openDataPlaceAccessBOOL=False, wikiDataBOOL=False)

        #儲存斷詞結果
        try:
            with open(resultFilePath, "w", encoding="utf-8") as resultFile:
                json.dump(resultDICT, resultFile, ensure_ascii=False)
                print("斷詞結果儲存成功")
        except Exception as e:
            print("斷詞結果儲存失敗：{}".format(e))

    print("\n斷詞結果：")
    pprint(resultDICT["result_segmentation"])
    print("\n標記結果：")
    pprint(resultDICT["result_pos"])

    #列出目前可使用的 Articut 版本選擇。通常版本號愈大，完成度愈高。
    versions = articut.versions()
    print("\n##Avaliable Versions:")
    pprint(versions)

    #列出所有的 content word.
    contentWordLIST = articut.getContentWordLIST(resultDICT)
    print("\n##ContentWord:")
    pprint(contentWordLIST)

    #列出所有的化學類名詞.
    chemicalLIST = articut.getChemicalLIST(resultDICT)
    print("\n##Chemical:")
    pprint(chemicalLIST)

    #列出所有的人名 (不含代名詞).
    personLIST = articut.getPersonLIST(resultDICT, includePronounBOOL=False)
    print("\n##Person (Without Pronoun):")
    pprint(personLIST)
    personLIST = articut.getPersonLIST(resultDICT, includePronounBOOL=True)
    print("\n##Person (With Pronoun):")
    pprint(personLIST)

    #列出所有的 verb word. (動詞)
    verbStemLIST = articut.getVerbStemLIST(resultDICT)
    print("\n##Verb:")
    pprint(verbStemLIST)

    #列出所有的 noun word. (名詞)
    nounStemLIST = articut.getNounStemLIST(resultDICT)
    print("\n##Noun:")
    pprint(nounStemLIST)

    #列出所有的 time (時間)
    timeLIST = articut.getTimeLIST(resultDICT)
    print("\n##Time:")
    pprint(timeLIST)

    #列出所有的 location word. (地方名稱)
    locationStemLIST = articut.getLocationStemLIST(resultDICT)
    print("\n##Location:")
    pprint(locationStemLIST)

    #允許 Articut 調用字典，列出所有政府開放資料中列為觀光地點名稱的字串。(地點名稱)
    placeLIST = articut.getOpenDataPlaceLIST(resultDICT)
    print("\n##Place:")
    pprint(placeLIST)

    #允許 Articut 調用 WikiData 字典，列出所有 WikiData 條目名稱的字串。
    wikiDataLIST = articut.getWikiDataLIST(resultDICT)
    print("\n##WikiData:")
    pprint(wikiDataLIST)

    #列出所有的 CLAUSE 問句
    questionLIST = articut.getQuestionLIST(resultDICT)
    print("\n##Question:")
    pprint(questionLIST)

    #列出所有的台灣地址
    addTWLIST = articut.getAddTWLIST(resultDICT)
    print("\n##Address:")
    pprint(addTWLIST)

    #使用 TF-IDF 演算法
    tfidfResult = articut.analyse.extract_tags(resultDICT)
    print("\n##TF-IDF:")
    pprint(tfidfResult)

    #使用 Textrank 演算法
    textrankResult = articut.analyse.textrank(resultDICT)
    print("\n##Textrank:")
    pprint(textrankResult)

    #使用 localRE 工具取得地址分段細節
    countyResult = articut.localRE.getAddressCounty(resultDICT)
    print("\n##localRE: 縣")
    pprint(countyResult)

    cityResult = articut.localRE.getAddressCity(resultDICT)
    print("\n##localRE: 市")
    pprint(cityResult)

    districtResult = articut.localRE.getAddressDistrict(resultDICT)
    print("\n##localRE: 區")
    pprint(districtResult)

    townshipResult = articut.localRE.getAddressTownship(resultDICT)
    print("\n##localRE: 鄉里")
    pprint(townshipResult)

    townResult = articut.localRE.getAddressTown(resultDICT)
    print("\n##localRE: 鎮")
    pprint(townResult)

    villageResult = articut.localRE.getAddressVillage(resultDICT)
    print("\n##localRE: 村")
    pprint(villageResult)

    neighborhoodResult = articut.localRE.getAddressNeighborhood(resultDICT)
    print("\n##localRE: 鄰")
    pprint(neighborhoodResult)

    roadResult = articut.localRE.getAddressRoad(resultDICT)
    print("\n##localRE: 路")
    pprint(roadResult)

    sectionResult = articut.localRE.getAddressSection(resultDICT)
    print("\n##localRE: 段")
    pprint(sectionResult)

    alleyResult = articut.localRE.getAddressAlley(resultDICT)
    print("\n##localRE: 巷、弄")
    pprint(alleyResult)

    numberResult = articut.localRE.getAddressNumber(resultDICT)
    print("\n##localRE: 號")
    pprint(numberResult)

    floorResult = articut.localRE.getAddressFloor(resultDICT)
    print("\n##localRE: 樓")
    pprint(floorResult)

    roomResult = articut.localRE.getAddressRoom(resultDICT)
    print("\n##localRE: 室")
    pprint(roomResult)

    #列出所有的貨幣金額
    currencyResult = articut.getCurrencyLIST(resultDICT)
    print("\n##currencyLIST:")
    pprint(currencyResult)

    inputSTR = "前天你說便宜的油還在海上，怎麼兩天後就到港口了？"
    lv3result = articut.parse(inputSTR, level="lv3",
                              userDefinedDictFILE=None,
                              openDataPlaceAccessBOOL=False,
                              wikiDataBOOL=False,
                              indexWithPOS=False,
                              timeRef=None,
                              pinyin="BOPOMOFO")
    pprint(lv3result)


try:
    from .ArticutAPI import Articut
    from .Toolkit.analyse import AnalyseManager
    from .Toolkit.localRE import TaiwanAddressAnalizer
    from .Toolkit.toolkits import *
    from .Toolkit.NER import GenericNER
except:
    from ArticutAPI import Articut
    from Toolkit.analyse import AnalyseManager
    from Toolkit.localRE import TaiwanAddressAnalizer
    from Toolkit.toolkits import *
    from Toolkit.NER import GenericNER



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import re

class TaiwanAddressAnalizer:
    def __init__(self, locale=None):
        if locale in ("TW", "CN"):
            pass
        else:
            locale="TW"
        self.addTWPat = re.compile("(?<=<KNOWLEDGE_addTW>)[^<]*?(?=</KNOWLEDGE_addTW>)")
        self.TWaddPatDICT = {"countyPat"      : "[^\s][^市]縣",
                             "cityPat"        : "[^是在於及、，\s]{1,2}市",
                             "districtPat"    : "那瑪夏區|[^市及、，\s]+?.社?區|[東西南北中]區",
                             "townshipPat"    : "(阿里山|三地門|太麻里)鄉|..[鄉里]",
                             "townPat"        : "[^\s][^\s]鎮",
                             "villagePat"     : "[^\s][^\s]村",
                             "neighborhoodPat": "(\s?[零一二三四五六七八九十廿卅０-９\d]+?\s?鄰)",
                             "roadPat"        : "市府路|市[政場]([北南中]|[1-7一二三四五六七]){0,2}路|市港[^\s]路|[新環村盛果]市[^\s]?[路街]|市民大道|市宅街|[埔菜美元西]市[路街]|(?<=[縣市區鄉鎮里村鄰])[^市區鄉鎮村路及鄰、，]{1,4}([路街]|大道)",
                             "sectionPat"     : "\s?[零一二三四五六七八九十廿卅百０-９\d]*?\s?段",
                             "alleyPat"       : "(國中|市場|新市.|([^縣市區鄉鎮里村路段]{1,2}|鐵路)[零一二三四五六七八九十廿卅百０-９\d]*?|\s?[零一二三四五六七八九十廿卅百０-９\d]*?)\s?巷([零一二三四五六七八九十廿卅百０-９\d]*?\s?弄)?",
                             "numberPat"      : "(\s?[零一二三四五六七八九十廿卅百０-９\d]*?\s?[之\-]\s?)?\s?[零一二三四五六七八九十廿卅百０-９\d]*?\s?號([之\-]\s?[零一二三四五六七八九十廿卅百０-９\d]+?)?",
                             "floorPat"       : "\s?[零一二三四五六七八九十廿卅百０-９\d]*?\s?[fF樓]",
                             "roomPat"        : "\s?([a-zA-Z\d０-９]+?)\s?(室?$)"}
        self.stripPat = re.compile("(?<=>).*?(?=<)")

    def _addIndexConverter(self, ArticutResultDICT, addIndexLIST):
        '''
        Convert posIndex to segIndex
        Return list
        '''
        if type(addIndexLIST) is list:
            pass
        else:
            return None
        segIndexLIST = []
        try:
            tagLen = len("<KNOWLEDGE_addTW>")
            for i, posLIST in enumerate(addIndexLIST):
                if posLIST:
                    tmpLIST = []
                    for start, end, seg in posLIST:
                        posEndSTR = ArticutResultDICT["result_pos"][i][:start]
                        endSTR = posEndSTR[posEndSTR.rfind("<KNOWLEDGE_addTW>")+tagLen:]
                        segEndSTR = "".join([x.group() for x in self.stripPat.finditer(posEndSTR)])
                        tmpLIST.append((len(segEndSTR+endSTR), len(segEndSTR+endSTR)+len(seg), seg))
                    segIndexLIST.append(tmpLIST)
                else:
                    segIndexLIST.append(posLIST)
        except Exception as e:
            print("Invalid posIndexLIST format")
            return None
        return segIndexLIST

    def _getAddLIST(self, ArticutResultDICT, addPatSTR):
        if "result_pos" in ArticutResultDICT:
            pass
        else:
            return None
        resultLIST = []
        addPat = re.compile(addPatSTR)
        for r in ArticutResultDICT["result_pos"]:
            if "<KNOWLEDGE_addTW>" in r:
                tmpLIST = [(c.start(), c.end(), c.group(0)) for c in list(self.addTWPat.finditer(r))]
                resultLIST.append([])
                for start, end, tmp in tmpLIST:
                    resultLIST[-1].extend([[c.start()+start, c.end()+start, c.group(0)] for c in list(addPat.finditer(tmp))])
            else:
                resultLIST.append([])
                continue
        return resultLIST

    def mergeBulkResult(self, inputLIST):
        resultLIST = []
        resultExtend = resultLIST.extend
        for x in filter(None, inputLIST):
            try:
                if x["status"]:    # 只取成功的結果
                    resultExtend(x["result_list"])
            except:
                pass
        return resultLIST

    def getAddressCounty(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["countyPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["countyPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressCity(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["cityPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["cityPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressDistrict(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["districtPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["districtPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressTownship(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["townshipPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["townshipPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressTown(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["townPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["townPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressVillage(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["villagePat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["villagePat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressNeighborhood(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["neighborhoodPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["neighborhoodPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressRoad(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["roadPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["roadPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressSection(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["sectionPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["sectionPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressAlley(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["alleyPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["alleyPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressNumber(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["numberPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["numberPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressFloor(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["floorPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["floorPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def getAddressRoom(self, ArticutResultDICT, indexWithPOS=True):
        resultLIST = []
        resultAppend = resultLIST.append
        if type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend(self._getAddLIST(x, self.TWaddPatDICT["roomPat"]))
                if not indexWithPOS and resultLIST:
                    resultLIST[i] = self._addIndexConverter(x, resultLIST)
        else:
            resultLIST = self._getAddLIST(ArticutResultDICT, self.TWaddPatDICT["roomPat"])
            if not indexWithPOS and resultLIST:
                resultLIST = self._addIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

try:
    import rapidjson as json
except:
    import json

import os
import re

def mergeBulkResult(inputLIST):
    resultLIST = []
    resultExtend = resultLIST.extend
    for x in filter(None, inputLIST):
        try:
            if x["status"]:    # 只取成功的結果
                resultExtend(x["result_list"])
        except:
            pass
    return resultLIST

class ArticutPOS:
    def __init__(self):
        # Regex Pattern
        self.verbPat = re.compile("(?<=<VerbP>)[^<]*?(?=.</VerbP>)|(?<=<ACTION_verb>)[^<]*?(?=</ACTION_verb>)")
        self.nounPat = re.compile("(?<=<ENTITY_nounHead>)[^<]*?(?=</ENTITY_nounHead>)|(?<=<ENTITY_nouny>)[^<]*?(?=</ENTITY_nouny>)|(?<=<ENTITY_noun>)[^<]*?(?=</ENTITY_noun>)|(?<=<ENTITY_oov>)[^<]*?(?=</ENTITY_oov>)")
        self.modifierPat = re.compile("(?<=<MODIFIER>)[^<]*?(?=</MODIFIER>)")
        self.modifierPPat = re.compile("(?<=<DegreeP>)[^<]*?(?=</DegreeP>)|(?<=<ModifierP>)[^<]*?(?=</ModifierP>)")
        self.funcPat = re.compile("(?<=<AUX>)[^<]*?(?=</AUX>)|(?<=<FUNC_in[nt]er>)[^<]*?(?=</FUNC_in[nt]er>)|(?<=<RANGE_locality>)[^<]*?(?=</RANGE_locality>)|(?<=<RANGE_period>)[^<]*?(?=</RANGE_period>)")
        self.personPat = re.compile("(?<=<ENTITY_person>)[^<]*?(?=</ENTITY_person>)")
        self.pronounPat = re.compile("(?<=<ENTITY_pronoun>)[^<]*?(?=</ENTITY_pronoun>)")
        self.locationPat = re.compile("(?<=<LOCATION>)[^<]*?(?=</LOCATION>)|(?<=<KNOWLEDGE_addTW>)[^<]*?(?=</KNOWLEDGE_addTW>)|(?<=<KNOWLEDGE_routeTW>)[^<]*?(?=</KNOWLEDGE_routeTW>)")
        self.userDefinedPat = re.compile("(?<=<UserDefined>)[^<]*?(?=</UserDefined>)")
        self.placePat = re.compile("(?<=<KNOWLEDGE_place>)[^<]*?(?=</KNOWLEDGE_place>)")
        self.timePat = re.compile("(?<=<TIME_decade>)[^<]*?(?=</TIME_decade>)|(?<=<TIME_year>)[^<]*?(?=</TIME_year>)|(?<=<TIME_season>)[^<]*?(?=</TIME_season>)|(?<=<TIME_month>)[^<]*?(?=</TIME_month>)|(?<=<TIME_week>)[^<]*?(?=</TIME_week>)|(?<=<TIME_day>)[^<]*?(?=</TIME_day>)|(?<=<TIME_justtime>)[^<]*?(?=</TIME_justtime>)")
        self.addTWPat = re.compile("(?<=<KNOWLEDGE_addTW>)[^<]*?(?=</KNOWLEDGE_addTW>)")
        self.colorPat = re.compile("(?<=<MODIFIER_color>)[^<]+?(?=</MODIFIER_color>)")
        self.currencyPat = re.compile("(?<=<KNOWLEDGE_currency>)[^<]*?(?=</KNOWLEDGE_currency>)")
        self.currencyGreedyPat = re.compile("(?<=[元金幣圜圓比布索鎊盾銖令朗郎]</ENTITY_noun><ENTITY_num>)[^<]*?(?=</ENTITY_num>)")
        self.currencyGreedyGapPat = re.compile("(?<=^<ENTITY_num>)[^<]*?(?=</ENTITY_num>)")
        self.chemicalPat = re.compile("(?<=<KNOWLEDGE_chemical>)[^<]*?(?=</KNOWLEDGE_chemical>)")
        self.wikiDataPat = re.compile("(?<=<KNOWLEDGE_wikiData>)[^<]*?(?=</KNOWLEDGE_wikiData>)")
        self.stripPat = re.compile("(?<=>).*?(?=<)")
        self.clausePat = re.compile("\<CLAUSE_.*?Q\>")
        self.contentPat = re.compile("|".join([self.verbPat.pattern, self.nounPat.pattern, self.modifierPat.pattern, self.modifierPPat.pattern, self.userDefinedPat.pattern]))

    def _segIndexConverter(self, parseResultDICT, posIndexLIST):
        '''
        Convert posIndex to segIndex
        Return list
        '''

        if type(posIndexLIST) is list and "result_pos" in parseResultDICT:
            pass
        else:
            return None

        segIndexLIST = []
        try:
            for i, posLIST in enumerate(posIndexLIST):
                if posLIST:
                    tmpLIST = []
                    for start, end, seg in posLIST:
                        posEndSTR = parseResultDICT["result_pos"][i][:start]
                        segEndSTR = "".join([x.group() for x in self.stripPat.finditer(posEndSTR)])
                        tmpLIST.append((len(segEndSTR), len(segEndSTR)+len(seg), seg))
                    segIndexLIST.append(tmpLIST)
                else:
                    segIndexLIST.append(posLIST)
        except Exception:
            print("Invalid posIndexLIST format")
            return None
        return segIndexLIST

    def getAddTWLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中含有 (KNOWLEDGE_addTW) 標籤的字串。
        該字串為一台灣地址。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        addTWLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                addTWLIST.append([(a.start(), a.end(), a.group(0)) for a in list(self.addTWPat.finditer(p))])
            else:
                addTWLIST.append([])
        if not indexWithPOS:
            addTWLIST = self._segIndexConverter(parseResultDICT, addTWLIST)
        return addTWLIST

    def getColorLIST(self, resultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中含有 (MODIFIER_color) 標籤的字串。
        該字串為一顏色表述字串
        '''
        if "result_pos" in resultDICT:
            pass
        else:
            return None
        colorLIST = []
        for p in resultDICT["result_pos"]:
            if len(p) > 1:
                colorLIST.append([(a.start(), a.end(), a.group(0)) for a in list(self.colorPat.finditer(p))])
            else:
                colorLIST.append([])
        if not indexWithPOS:
            colorLIST = self._segIndexConverter(resultDICT, colorLIST)
        return colorLIST

    def getCurrencyLIST(self, parseResultDICT, indexWithPOS=True, greedyBOOL=False):
        '''
        取出斷詞結果中的貨幣金額 (KNOWLEDGE_currency) 標籤的字串。
        每個句子內的「貨幣金額」，將列為一個 list。
        若 greedy = True，則以下格式會加到回傳 list
            貨幣名稱 + 數字 (包含「'」與「,」符號)
            新台幣 100
            美金9.99
            歐元 1,999'99
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        currencyLIST = []
        for i, p in enumerate(parseResultDICT["result_pos"]):
            if len(p) > 1:
                currencyLIST.append([(c.start(), c.end(), c.group(0)) for c in list(self.currencyPat.finditer(p))])
                if greedyBOOL:
                    greedyLIST = []
                    try:
                        if parseResultDICT["result_pos"][i-1][-14:] == "</ENTITY_noun>" and parseResultDICT["result_pos"][i-1][-15] in "元金幣圜圓比布索鎊盾銖令朗郎":
                            greedyLIST = [(c.start(), c.end(), c.group(0)) for c in list(self.currencyGreedyGapPat.finditer(p))]
                    except:
                        pass
                    if greedyLIST:
                        greedyLIST.extend([(c.start(), c.end(), c.group(0)) for c in list(self.currencyGreedyPat.finditer(p))])
                    else:
                        greedyLIST = [(c.start(), c.end(), c.group(0)) for c in list(self.currencyGreedyPat.finditer(p))]
                    if greedyLIST:
                        currencyLIST[-1].extend(greedyLIST)
            else:
                currencyLIST.append([])
        if not indexWithPOS:
            currencyLIST = self._segIndexConverter(parseResultDICT, currencyLIST)
        return currencyLIST

    def getContentWordLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的實詞 (content word)。
        每個句子內的實詞為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        contentWordLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                contentWordLIST.append([(c.start(), c.end(), c.group(0)) for c in list(self.contentPat.finditer(p))])
            else:
                contentWordLIST.append([])
        if not indexWithPOS:
            contentWordLIST = self._segIndexConverter(parseResultDICT, contentWordLIST)
        return contentWordLIST

    def getChemicalLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的 KNOWLEDGE_chemical。
        每個句子內的 KNOWLEDGE_chemical 為一個 list.
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        chemicalLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                chemicalLIST.append([(c.start(), c.end(), c.group(0)) for c in list(self.chemicalPat.finditer(p))])
            else:
                chemicalLIST.append([])
        if not indexWithPOS:
            chemicalLIST = self._segIndexConverter(parseResultDICT, chemicalLIST)
        return chemicalLIST

    def getLocationStemLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的地理位置 (location)。此處指的是地理位置標記的行政區地名詞彙，例如「台北」、「桃園」、「墨西哥」。
        每個句子內的地理位置列為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        locationLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                locationLIST.append([(l.start(), l.end(), l.group(0)) for l in list(self.locationPat.finditer(p))])
            else:
                locationLIST.append([])
        if not indexWithPOS:
            locationLIST = self._segIndexConverter(parseResultDICT, locationLIST)
        return locationLIST

    def getNounStemLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的名詞 (noun)。此處指的是 ENTITY_noun、ENTITY_nouny、ENTITY_nounHead 或 ENTITY_oov 標記的名詞詞彙。
        每個句子內的名詞為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        nounLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                nounLIST.append([(n.start(), n.end(), n.group(0)) for n in list(self.nounPat.finditer(p))])
            else:
                nounLIST.append([])
        if not indexWithPOS:
            nounLIST = self._segIndexConverter(parseResultDICT, nounLIST)
        return nounLIST

    def getOpenDataPlaceLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的景點 (KNOWLEDGE_place) 標籤的字串。此處指的是景點 (KNOWLEDGE_place)標記的非行政地點名稱詞彙，例如「鹿港老街」、「宜蘭運動公園」。
        每個句子內的景點為一個 list.
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None

        placeLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                placeLIST.append([(l.start(), l.end(), l.group(0)) for l in list(self.placePat.finditer(p))])
            else:
                placeLIST.append([])
        if not indexWithPOS:
            placeLIST = self._segIndexConverter(parseResultDICT, placeLIST)
        return placeLIST

    def getPersonLIST(self, parseResultDICT, includePronounBOOL=True, indexWithPOS=True):
        '''
        取出斷詞結果中的人名 (Person)
        若 includePronounBOOL 為 True，則連代名詞 (Pronoun) 一併回傳；若為 False，則只回傳人名。
        回傳結果為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        person_pronounLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p)>1:
                personLIST = [(pn.start(), pn.end(), pn.group(0)) for pn in list(self.personPat.finditer(p))]
                person_pronounLIST.append(personLIST)
            else:
                person_pronounLIST.append([])
        if includePronounBOOL == True:
            for p in parseResultDICT["result_pos"]:
                if len(p)==1:
                    pass
                else:
                    person_pronounLIST[parseResultDICT["result_pos"].index(p)].extend([(pn.start(), pn.end(), pn.group(0)) for pn in list(self.pronounPat.finditer(p))])
        if not indexWithPOS:
            person_pronounLIST = self._segIndexConverter(parseResultDICT, person_pronounLIST)
        return person_pronounLIST

    def getQuestionLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中含有 (CLAUSE_Q) 標籤的句子。
        此處指的是
            <CLAUSE_AnotAQ>: A-not-A 問句
            <CLAUSE_YesNoQ>: 是非問句
            <CLAUSE_WhoQ">: 「誰」問句
            <CLAUSE_WhatQ>: 「物」問句
            <CLAUSE_WhereQ>: 「何地」問句
            <CLAUSE_WhenQ>: 「何時」問句
            <CLAUSE_HowQ>: 「程度/過程」問句
            <CLAUSE_WhyQ>: 「原因」問句
        每個句子內若有 <CLAUSE_Q> 標籤，整個句子將會存進 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None

        questionLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                tmpLIST = [q for q in list(self.clausePat.finditer(p))]
                if tmpLIST:
                    for q in tmpLIST:
                        questionLIST.append([(q.group(0), "".join([x.group(0) for x in self.stripPat.finditer(p)])) for q in list(self.clausePat.finditer(p))])
                else:
                    questionLIST.append([])
            else:
                questionLIST.append([])
        if not indexWithPOS:
            questionLIST = self._segIndexConverter(parseResultDICT, questionLIST)
        return questionLIST

    def getTimeLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的時間 (time)。
        每個句子內的「時間」詞列為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        timeLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                timeLIST.append([(l.start(), l.end(), l.group(0)) for l in list(self.timePat.finditer(p))])
            else:
                timeLIST.append([])
        if not indexWithPOS:
            timeLIST = self._segIndexConverter(parseResultDICT, timeLIST)
        return timeLIST

    def getVerbStemLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的動詞 (verb)。此處指的是 ACTION_verb 標記的動詞詞彙。
        每個句子內的動詞為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        verbLIST = []

        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                verbLIST.append([(v.start(), v.end(), v.group(0)) for v in list(self.verbPat.finditer(p))])
            else:
                verbLIST.append([])
        if not indexWithPOS:
            verbLIST = self._segIndexConverter(parseResultDICT, verbLIST)
        return verbLIST

    def getWikiDataLIST(self, parseResultDICT, indexWithPOS=True):
        '''
        取出斷詞結果中的 WikiData 標記文字。此處指的是 KNOWLEDGE_wikiData 標記的條目名稱。
        每個句子內的條目名稱為一個 list。
        '''
        if "result_pos" in parseResultDICT:
            pass
        else:
            return None
        wikiDataLIST = []
        for p in parseResultDICT["result_pos"]:
            if len(p) > 1:
                wikiDataLIST.append([(n.start(), n.end(), n.group(0)) for n in list(self.wikiDataPat.finditer(p))])
            else:
                wikiDataLIST.append([])
        if not indexWithPOS:
            wikiDataLIST = self._segIndexConverter(parseResultDICT, wikiDataLIST)
        return wikiDataLIST

class LawsToolkit:
    def __init__(self, articutResult=None):
        self.articutResult = articutResult
        self.articlePat = re.compile("<KNOWLEDGE_lawTW>[^<]+?</KNOWLEDGE_lawTW>")
        self.crimePat = re.compile("(?<=[犯判決]</ACTION_verb>)[^犯罪]*?(>(?=<KNOWLEDGE_lawTW>第)|罪(?=<))")
        self.criminalResponsibilityPat = re.compile("(?<=<ACTION_verb>處</ACTION_verb>)[^處極重]*?[刑役](?=<)(<[^>]*?>)?(<TIME_year>[^<]+?</TIME_year>)?(<TIME_month>[^<]+?</TIME_month>)?")
        self.eventRefPat = re.compile("<FUNC_inner>所</FUNC_inner><ACTION_lightVerb>受</ACTION_lightVerb>(<FUNC_inner>之</FUNC_inner>)?(<QUANTIFIER>[^<]+?</QUANTIFIER>)?<ENTITY_nouny>..</ENTITY_nouny>")

    def tagPurger(self, posSTR):
        textSTR = re.sub("<[^<]*?>", "", posSTR)
        return textSTR

    def getLawArticle(self, parseResultDICT={}):
        '''
        取得法條編號
        '''
        if parseResultDICT:
            self.articutResult = parseResultDICT
        articleLIST = []
        if type(self.articutResult) is list:
            self.articutResult = mergeBulkResult(self.articutResult)
            articleAppend = articleLIST.append
            for x in self.articutResult:
                articleAppend(list(set([self.tagPurger(a.group(0)) for a in self.articlePat.finditer("".join(x["result_pos"]))])))
        else:
            articleLIST = list(set([self.tagPurger(a.group(0)) for a in self.articlePat.finditer("".join(self.articutResult["result_pos"]))]))
        return articleLIST

    def getCrime(self, parseResultDICT={}):
        '''
        取得罪名
        '''
        if parseResultDICT:
            self.articutResult = parseResultDICT

        crimeTextLIST = []
        if type(self.articutResult) is list:
            self.articutResult = mergeBulkResult(self.articutResult)
            crimeTextAppend = crimeTextLIST.append
            for x in self.articutResult:
                crimePosLIST = set([c.group(0) for c in self.crimePat.finditer("".join(x["result_pos"]))])
                crimeTextAppend([self.tagPurger(c) for c in crimePosLIST])
        else:
            crimePosLIST = set([c.group(0) for c in self.crimePat.finditer("".join(self.articutResult["result_pos"]))])
            crimeTextLIST = [self.tagPurger(c) for c in crimePosLIST]
        return crimeTextLIST

    def getCriminalResponsibility(self, parseResultDICT={}):
        '''
        取得刑責
        To be deprecated soon.
        '''
        if parseResultDICT:
            self.articutResult = parseResultDICT
        try:
            crTextLIST = []
            if type(self.articutResult) is list:
                self.articutResult = mergeBulkResult(self.articutResult)
                crTextAppend = crTextLIST.append
                for x in self.articutResult:
                    crPosLIST = set([c.group(0) for c in self.criminalResponsibilityPat.finditer("".join(x["result_pos"]))])
                    crTextAppend([self.tagPurger(c) for c in crPosLIST])
            else:
                crPosLIST = set([c.group(0) for c in self.criminalResponsibilityPat.finditer("".join(self.articutResult["result_pos"]))])
                crTextLIST = [self.tagPurger(c) for c in crPosLIST]
            print("getCriminalResponsibility() To be deprecated soon.")
            return crTextLIST
        except KeyError:
            return []

    def getPenalty(self, parseResultDICT={}):
        '''
        取得刑責。
        Dummy Function of getCriminalResponsibility()
        '''
        return self.getCriminalResponsibility(parseResultDICT)

    def getEventRef(self, parseResultDICT={}):
        '''
        取得「所受(之)XX」的列表
        '''
        if parseResultDICT:
            self.articutResult = parseResultDICT

        erTextLIST = []
        if type(self.articutResult) is list:
            self.articutResult = mergeBulkResult(self.articutResult)
            erTextAppend = erTextLIST.append
            for x in self.articutResult:
                erPosLIST = set([e.group(0) for e in self.eventRefPat.finditer("".join(x["result_pos"]))])
                erTextAppend([self.tagPurger(e) for e in erPosLIST])
        else:
            erPosLIST = set([e.group(0) for e in self.eventRefPat.finditer("".join(self.articutResult["result_pos"]))])
            erTextLIST = [self.tagPurger(e) for e in erPosLIST]

        return erTextLIST

class UserDefinedDictToolkit:
    def __init__(self):
        self.msg_AticutDictERROR = "\n".join(["請輸入 Articut.parse() 處理完的 dictionary 做為參數！", "Please specify dictionary returned by Articut.parse() as argument!"])
        self.msg_UserDefinedDictionaryDIRERROR = "\n".join(["", ""])

    def tagByDictName(self, ArticutDICT, UserDefinedDictionaryDIR):
        try:
            if isinstance(ArticutDICT, dict):
                if "result_pos" in ArticutDICT:
                    pass
                else:
                    return self.msg_AticutDictERROR
            else:
                return self.msg_AticutDictERROR
        except:
            return self.msg_AticutDictERROR

        dictLIST = [d for d in os.listdir(UserDefinedDictionaryDIR) if d.endswith(".json")]
        for D in dictLIST:
            try:
                dictName = D.split("/")[-1].split(".json")[0]
                with open(D) as f:
                    dDICT = json.load(f.read())
                    dLIST = []
                    for d in dDICT:
                        dLIST.append(d)
                        dLIST.extend(dDICT[d])
            except:
                return self.msg_UserDefinedDictionaryDIRERROR

            resultLIST = []
            for s in ArticutDICT["result_pos"]:
                for d in dLIST:
                    if "<UserDefined>{}</UserDefined>".format(d) in s:
                        resultLIST.append(s.replace("<UserDefined>{}</UserDefined>".format(d), "UD_{}".format(dictName)))
                    else:
                        resultLIST.append(s)
            ArticutDICT["result_pos"] = resultLIST
        return ArticutDICT

class Tokenizer:
    def __init__(self, articutResult):
        self.text = []
        self.tag_ = []
        self.idx = []
        self.pos_ = []



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import re

class GenericNER:
    def __init__(self, locale=None):
        self.mainDishSTR = "([乳包味啡嗲奶心手排果柳棗椒油湯漿炒焿煎煙燒爐瓜皮盅盤米粄粉粑粥粽粿糊糍粑糕糖糬糰線羹翅翼肉肚肺肝膽腐腦腩腳腸胃腿胗芛薏花茄茶草莓菊菜蓮蔗蔥薑蒜薩薯蘆蕉蜜筍檳豆豉栗菇虱蚵蛋卵蝦蟹螺貝蟳血角酒酥醬趖銼鍋雜頭飩飯飴餃飩餅麵麩餐骨鬆鬚魚鯧鰻鮪鮭魽鯛鰾鱉龜雞鴨鵝牛豬羊兔鼠鴿豚鷄堡]|"
        self.sideDishSTR = "[丸冰圓塊卷捲粄條扒仔乾]|[俄德法美日義英西歐中台泰越韓粵港]式|星州|客家|花枝|鴛鴦|石斑|黑輪|伯勞|藥燉|愛玉|珍珠|熱狗|阿給|燒賣|料理|飲料|糖[水醋]|鳳梨|蜜餞|花生|[壽土吐]司|便當|飯盒|鍋貼|扁食|雲吞|刺身|山蘇|多士|咖哩|沙拉|沙母|沙公|海參|鴨賞|旗魚|燒賣|海鮮|河鮮|龍珠|鮟鱇|牛河|熊掌|糕渣|自助餐|沙威[馬瑪]|三明治|沙淇瑪|原住民|棺材板|官財板|地瓜球|綠豆椪|冰淇淋|佛跳牆|[洋和][菓果]子)"
        self.cookMethodSTR = "[煎煮燉炒炸蒸煨魯滷漬醃炊爆燒拌剝湯焢焗烘烤佐釀涮火]"
        self.cookModSTR = "([麻辣酸清]|酥[脆炸]|[蔥薑蒜]爆|[干乾紅]燒){1,2}"
        self.extenedLIST = [("串燒", "(<ACTION_verb>串燒</ACTION_verb>)"),
                            ("烤肉", "(<ACTION_verb>烤肉</ACTION_verb>)"),
                            ("切雞", "(<VerbP>切雞</VerbP>)"),
                            ("炒蛋", "(<VerbP>炒蛋</VerbP>)"),
                            ("煎蛋", "(<VerbP>煎蛋</VerbP>)"),
                            ("烘蛋", "(<VerbP>烘蛋</VerbP>)"),
                            ("炒蝦", "(<VerbP>炒蝦</VerbP>)"),
                            ("叉燒", "(<ENTITY_oov>叉</ENTITY_oov><ACTION_verb>燒</ACTION_verb>)"),
                            ("卷煎", "(<ENTITY_oov>卷</ENTITY_oov><ACTION_verb>煎</ACTION_verb>)"),
                            ("阿拜", "(<ENTITY_nouny>阿</ENTITY_nouny><ACTION_verb>拜</ACTION_verb>)"),
                            ("吉拿富", "(<MODIFIER>吉</MODIFIER><ACTION_verb>拿富</ACTION_verb>)"),
                            #<日式食物名：動詞在後>
                            ("手捲", "(<ENTITY_oov>手</ENTITY_oov><ACTION_verb>捲</ACTION_verb>)"),
                            ("犬首燒", "(<ENTITY_oov>犬首</ENTITY_oov><ACTION_verb>燒</ACTION_verb>)"),
                            ("銅鑼燒", "(<ENTITY_oov>銅鑼</ENTITY_oov><ACTION_verb>燒</ACTION_verb>)"),
                            ("大雕燒", "(<ENTITY_nouny>大雕</ENTITY_nouny><ACTION_verb>燒</ACTION_verb>)"),
                            ("貓掌燒", "(<ENTITY_oov>貓掌</ENTITY_oov><ACTION_verb>燒</ACTION_verb>)"),
                            ("關東煮", "(<ENTITY_noun>關東煮</ENTITY_noun>)"),
                            ("廣島燒", "(<ENTITY_noun>廣島燒</ENTITY_noun>)"),
                            ("大阪燒", "(<ENTITY_noun>大阪燒</ENTITY_noun>)"),
                            ("雞白湯拉麵", "(<ENTITY_oov>雞</ENTITY_oov><MODIFIER_color>白</MODIFIER_color><ENTITY_nounHead>湯拉麵</ENTITY_nounHead>)"),
                            ("漢堡排", "(<ENTITY_oov>漢堡</ENTITY_oov><ACTION_verb>排</ACTION_verb>)"),
                            #</日式食物名：動詞在後>
                            ("宮保", "(<ENTITY_oov>宮</ENTITY_oov><ACTION_verb>保</ACTION_verb>)"),
                            ("西多士", "(<ENTITY_oov>西</ENTITY_oov><ENTITY_nouny>多士</ENTITY_nouny>)"),
                            ("漢堡包", "(<ENTITY_oov>漢堡</ENTITY_oov><ACTION_verb>包</ACTION_verb>)"),
                            ("黑貓包", "(<ENTITY_nounHead>黑貓</ENTITY_nounHead><ACTION_verb>包</ACTION_verb>)"),
                            ("飯包", "(<ENTITY_oov>飯</ENTITY_oov><ACTION_verb>包</ACTION_verb>)"),
                            ("煎釀三寶", "(<ACTION_verb>煎釀</ACTION_verb><ENTITY_num>三</ENTITY_num><ENTITY_oov>寶</ENTITY_oov>)"),
                            ("乾炒牛河", "(<ACTION_verb>乾</ACTION_verb><ACTION_verb>炒</ACTION_verb><ENTITY_nouny>牛河</ENTITY_nouny>)"),
                            ("星州炒米", "(<ENTITY_oov>星洲</ENTITY_oov><VerbP>炒米</VerbP>)"),
                            ("醃腸熟肉", "(<VerbP>醃腸</VerbP><ENTITY_nouny>熟肉</ENTITY_nouny>)"),
                            ("三文治", "(<ENTITY_num>三</ENTITY_num><ENTITY_oov>文</ENTITY_oov><ACTION_verb>治</ACTION_verb>)"),
                            ("五柳枝魚", "(<ENTITY_num>五</ENTITY_num><ENTITY_nouny>柳枝魚</ENTITY_nouny>)"),
                            ("浮水虱目魚羹", "(<VerbP>浮水</VerbP><ENTITY_noun>虱目魚羹</ENTITY_noun>)"),
                            ("咕咾肉", "(<ENTITY_oov>咕咾</ENTITY_oov><ENTITY_nounHead>肉</ENTITY_nounHead>)"),
                            ("三杯中卷", "(<ENTITY_classifier>三杯</ENTITY_classifier><RANGE_locality>中</RANGE_locality><ENTITY_oov>卷</ENTITY_oov>)"),
                            ("三杯小卷", "(<ENTITY_classifier>三杯</ENTITY_classifier><ENTITY_nouny>小卷</ENTITY_nouny>)"),
                            ("五更腸旺", "(<TIME_justtime>五更</TIME_justtime><ENTITY_oov>腸旺</ENTITY_oov>)"),
                            ("下水湯", "(<ACTION_verb>下水</ACTION_verb><ENTITY_nouny>湯</ENTITY_nouny>)"),
                            ("西魯肉", "(<ENTITY_oov>西魯</ENTITY_oov><ENTITY_nounHead>肉</ENTITY_nounHead>)"),
                            ("九層塔", "(<ENTITY_classifier>九層</ENTITY_classifier><ENTITY_nouny>塔</ENTITY_nouny>)"),
                            ("青蛙下蛋", "(<ENTITY_nounHead>青蛙</ENTITY_nounHead><ACTION_verb>下蛋</ACTION_verb>)"),
                            ("春卷", "(<TIME_season>春</TIME_season><ENTITY_nouny>卷</ENTITY_nouny>)"),
                            ("黑白切", "(<MODIFIER_color>黑</MODIFIER_color><MODIFIER_color>白</MODIFIER_color><ACTION_verb>切</ACTION_verb>)"),
                            ("臘八粥", "(<ENTITY_oov>臘</ENTITY_oov><ENTITY_num>八</ENTITY_num><ENTITY_nounHead>粥</ENTITY_nounHead>)"),
                            ("八寶粥", "(<ENTITY_num>八</ENTITY_num><ENTITY_nouny>寶粥</ENTITY_nouny>)"),
                            ("八寶冬粉", "(<ENTITY_num>八</ENTITY_num><ENTITY_oov>寶冬粉</ENTITY_oov>)"),
                            ("五色冷盤", "(<ENTITY_num>五</ENTITY_num><ENTITY_oov>色</ENTITY_oov><ENTITY_nouny>冷盤</ENTITY_nouny>)"),
                            ("蒼蠅頭", "(<ENTITY_oov>蒼蠅頭</ENTITY_oov>)"),
                            ("炒餌塊", "(<ACTION_verb>炒</ACTION_verb><ENTITY_nouny>餌塊</ENTITY_nouny>)"),
                            ("餌絲", "(<ENTITY_oov>餌</ENTITY_oov><ENTITY_classifier>絲</ENTITY_classifier>)"),
                            ("大救駕", "(<MODIFIER>大</MODIFIER><ACTION_verb>救駕</ACTION_verb>)"),
                            ("螞蟻上樹", "(<ENTITY_oov>螞蟻</ENTITY_oov><ACTION_verb>上樹</ACTION_verb>)"),
                            ("韭菜盒子", "(<ENTITY_oov>韭菜</ENTITY_oov><ENTITY_noun>盒子</ENTITY_noun>)"),
                            ("黯然銷魂飯", "(<MODIFIER>黯然</MODIFIER><ENTITY_nounHead>銷魂飯</ENTITY_nounHead>)")
                            ]
        self.escapeTUPLE = ("原住民", "客家", "俄式", "德式", "法式", "美式", "日式", "義式", "英式", "西式", "歐式", "中式", "台式", "泰式", "越式", "韓式", "粵式", "港式", "星州", "大塊", "菜單", "味")

        self.foodPat = re.compile("{0}|(<ACTION_verb>[^<]*?{1}+?[^<]*?</ACTION_verb>)?(<MODIFIER>{2}</MODIFIER>)?(<KNOWLEDGE_chemical>酸</KNOWLEDGE_chemical>)?(<MODIFIER_color>[黑紅]</MODIFIER_color>)?(<ENTITY_classifier>三杯</ENTITY_classifier>)?((<ENTITY_nounHead>[^<]*?{3}+[^<民]*?</ENTITY_nounHead>)|(<ENTITY_nouny>[^<]*?{3}+[^<民]*?</ENTITY_nouny>)|(<ENTITY_noun>[^<]*?{3}+[^<民]*?</ENTITY_noun>)|(<ENTITY_oov>[^<]*?{3}+[^<民]*?</ENTITY_oov>|<VerbP>{1}{3}</VerbP>))".format("|".join([p[1] for p in self.extenedLIST]), self.cookMethodSTR, self.cookModSTR, self.mainDishSTR+self.sideDishSTR))
        self.foodPatWLoc = re.compile(r"""{0}|
        (<LOCATION>[^<]+?</LOCATION>)?
        ((<ENTITY_nounHead>[^<]*?({1})+[^<]*?</ENTITY_nounHead>)|(<ENTITY_nouny>[^<]*?({1})+[^<]*?</ENTITY_nouny>)|(<ENTITY_noun>[^<]*?({1})+[^<]*?</ENTITY_noun>)|(<ENTITY_oov>[^<]*?({1})+[^<]*?</ENTITY_oov>))?
        (<ACTION_verb>[^<]*?{2}+?[^<]*?</ACTION_verb>)?
        (<MODIFIER>{3}</MODIFIER>)?
        (<KNOWLEDGE_chemical>酸</KNOWLEDGE_chemical>)?
        (<MODIFIER_color>[黑紅]</MODIFIER_color>)?
        (<ENTITY_classifier>三杯</ENTITY_classifier>)?
        ((<ENTITY_nounHead>[^<]*?{4}+[^<民]*?</ENTITY_nounHead>)|(<ENTITY_nouny>[^<]*?{4}+[^<民]*?</ENTITY_nouny>)|(<ENTITY_noun>[^<]*?{4}+[^<民]*?</ENTITY_noun>)|(<ENTITY_oov>[^<民]*?{4}+[^<]*?</ENTITY_oov>))""".format("|".join([p[1] for p in self.extenedLIST]),
                                                                                                                                                                                                                           "|".join(self.escapeTUPLE[:-2]),
                                                                                                                                                                                                                           self.cookMethodSTR,
                                                                                                                                                                                                                           self.cookModSTR,
                                                                                                                                                                                                                           self.mainDishSTR+self.sideDishSTR), re.X)

        self.agePat         = None
        self.anglePat       = None
        self.capacityPat    = None
        self.fractionPat    = None
        self.frequencyPat   = None
        self.lengthPat      = None
        self.locationPat    = None
        self.measurePat     = None
        self.percentPat     = None
        self.ratePat        = None
        self.speedPat       = None
        self.temperaturePat = None
        self.weightPat      = None
        self.areaPat        = None
        self.datePat        = None
        self.timePat        = None
        self.durationPat    = None
        self.integerPat     = None
        self.decimalPat     = None
        self.ordinalPat     = None
        self.currencyPat = re.compile("(?<=<KNOWLEDGE_currency>)[^<]*?(?=</KNOWLEDGE_currency>)")
        self.currencyGreedyPat = re.compile("(?<=[元金幣圜圓比布索鎊盾銖令朗郎]</ENTITY_noun><ENTITY_num>)[^<]*?(?=</ENTITY_num>)")
        self.currencyGreedyGapPat = re.compile("(?<=^<ENTITY_num>)[^<]*?(?=</ENTITY_num>)")
        self.personPat      = None
        self.pronounPat     = None
        self.organizationPat = None
        self.wwwPat         = None

        self.stripPat = re.compile("</?[A-Za-z]+?_?[A-Za-z]*?>")
        self.stripPatNAIVE = re.compile("(?<=>).*?(?=<)")

    def _segIndexConverter(self, ArticutResultDICT, posIndexLIST):
        '''
        Convert posIndex to segIndex. Token can be crossed.
        Return list
        '''
        if type(posIndexLIST) is list and "result_pos" in ArticutResultDICT:
            pass
        else:
            return None
        try:
            for sentenceINDEX in range(0, len(ArticutResultDICT["result_pos"])):
                if len(ArticutResultDICT["result_pos"][sentenceINDEX]) == 1: # 略過標點符號
                    pass
                else:
                    sentenceSTR = self.stripPat.sub("", ArticutResultDICT["result_pos"][sentenceINDEX])
                    refINDEX = 0
                    for i in posIndexLIST[sentenceINDEX]:
                        startINDEX = sentenceSTR[refINDEX:].find(i[2])
                        endINDEX   = startINDEX + len(i[2])
                        i[0] = refINDEX + startINDEX
                        i[1] = refINDEX + endINDEX
                        refINDEX = i[1]
        except Exception:
            print("Invalid posIndexLIST format")
            return None
        return posIndexLIST

    def _segIndexConverterNAIVE(self, ArticutResultDICT, posIndexLIST):
        if type(posIndexLIST) is list and "result_pos" in ArticutResultDICT:
            pass
        else:
            return None

        segIndexLIST = []
        try:
            for i, posLIST in enumerate(posIndexLIST):
                if posLIST:
                    tmpLIST = []
                    for start, end, seg in posLIST:
                        posEndSTR = ArticutResultDICT["result_pos"][i][:start]
                        segEndSTR = "".join([x.group() for x in self.stripPatNAIVE.finditer(posEndSTR)])
                        tmpLIST.append([len(segEndSTR), len(segEndSTR)+len(seg), seg])
                    segIndexLIST.append(tmpLIST)
                else:
                    segIndexLIST.append(posLIST)
        except Exception:
            print("Invalid posIndexLIST format")
            return None
        return segIndexLIST

    def _getFood(self, ArticutResultDICT, withLocation=False, indexWithPOS=True):

        resultLIST = []
        if withLocation == False:
            self.pat = self.foodPat
        else:
            self.pat = self.foodPatWLoc

        for p in ArticutResultDICT["result_pos"]:
            if len(p) > 1:
                resultLIST.append([[f.start(), f.end(), self.stripPat.sub("", f.group(0))] for f in list(self.pat.finditer(p)) if self.stripPat.sub("", f.group(0)) not in self.escapeTUPLE])
            else:
                resultLIST.append([])

        for sentenceINDEX in range(0, len(resultLIST)):
            for foodINDEX in range(0, len(resultLIST[sentenceINDEX])-1):
                if resultLIST[sentenceINDEX][foodINDEX][0] != None:
                    if resultLIST[sentenceINDEX][foodINDEX][1] == resultLIST[sentenceINDEX][foodINDEX+1][0]:
                        resultLIST[sentenceINDEX][foodINDEX][1] = resultLIST[sentenceINDEX][foodINDEX+1][1]
                        resultLIST[sentenceINDEX][foodINDEX][2] = resultLIST[sentenceINDEX][foodINDEX][2] + resultLIST[sentenceINDEX][foodINDEX+1][2]

                        resultLIST[sentenceINDEX][foodINDEX+1][0] = None
                        try:
                            if resultLIST[sentenceINDEX][foodINDEX][1] == resultLIST[sentenceINDEX][foodINDEX+2][0]:
                                resultLIST[sentenceINDEX][foodINDEX][1] = resultLIST[sentenceINDEX][foodINDEX+2][1]
                                resultLIST[sentenceINDEX][foodINDEX][2] = resultLIST[sentenceINDEX][foodINDEX][2] + resultLIST[sentenceINDEX][foodINDEX+2][2]

                                resultLIST[sentenceINDEX][foodINDEX+2][0] = None
                        except:
                            pass
        resultLIST = [[food for food in sentence if food[0]!=None] for sentence in resultLIST]

        if not indexWithPOS:
            resultLIST = self._segIndexConverter(ArticutResultDICT, resultLIST)
        return resultLIST

    def _getMoney(self, ArticutResultDICT, greedyBOOL=False, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「金額」的字串
        此功能和 ArticutAPI 中的 getCurrencyLIST() 等效。
        '''
        if "result_pos" in ArticutResultDICT:
            pass
        else:
            return None
        currencyLIST = []
        for i, p in enumerate(ArticutResultDICT["result_pos"]):
            if len(p) > 1:
                currencyLIST.append([(c.start(), c.end(), c.group(0)) for c in list(self.currencyPat.finditer(p))])
                if greedyBOOL:
                    greedyLIST = []
                    try:
                        if ArticutResultDICT["result_pos"][i-1][-14:] == "</ENTITY_noun>" and ArticutResultDICT["result_pos"][i-1][-15] in "元金幣圜圓比布索鎊盾銖令朗郎":
                            greedyLIST = [(c.start(), c.end(), c.group(0)) for c in list(self.currencyGreedyGapPat.finditer(p))]
                    except:
                        pass
                    if greedyLIST:
                        greedyLIST.extend([(c.start(), c.end(), c.group(0)) for c in list(self.currencyGreedyPat.finditer(p))])
                    else:
                        greedyLIST = [(c.start(), c.end(), c.group(0)) for c in list(self.currencyGreedyPat.finditer(p))]
                    if greedyLIST:
                        currencyLIST[-1].extend(greedyLIST)
            else:
                currencyLIST.append([])
        if not indexWithPOS:
            currencyLIST = self._segIndexConverterNAIVE(ArticutResultDICT, currencyLIST)
        return currencyLIST

    def _getMSRA(self, ArticutResultDICT, msraPat, indexWithPOS=True):

        resultLIST = []
        resultAppend = resultLIST.append

        if "result_pos" in ArticutResultDICT:
            for p in ArticutResultDICT["result_pos"]:
                if len(p) > 1:
                    resultAppend([[f.start(), f.end(), self.stripPat.sub("", f.group(0))] for f in msraPat.finditer(p)])
                else:
                    resultAppend([])
            if not indexWithPOS:
                resultLIST = self._segIndexConverter(ArticutResultDICT, resultLIST)
        elif type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend([])
                for p in x["result_pos"]:
                    if len(p) > 1:
                        resultLIST[i].append([[f.start(), f.end(), self.stripPat.sub("", f.group(0))] for f in msraPat.finditer(p)])
                    else:
                        resultLIST[i].append([])
                if not indexWithPOS:
                    resultLIST[i] = self._segIndexConverter(x, resultLIST[i])
        else:
            return None

        return resultLIST

    def mergeBulkResult(self, inputLIST):
        resultLIST = []
        resultExtend = resultLIST.extend
        for x in filter(None, inputLIST):
            try:
                if x["status"]:    # 只取成功的結果
                    resultExtend(x["result_list"])
            except:
                pass
        return resultLIST

    def getAge(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的「歲數」字串
        '''
        if self.agePat !=None:
            pass
        else:
            self.agePat = re.compile("<ENTITY_num>[^<]+?</ENTITY_num><ENTITY_noun>歲</ENTITY_noun>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.agePat, indexWithPOS)

        return resultLIST

    def getAngle(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「角度」的字串
        '''
        if self.anglePat !=None:
            pass
        else:
            self.anglePat = re.compile("(?<!燒</ACTION_verb>)(?<![溫氏]</ENTITY_nounHead>)(?<![溫氏]</ENTITY_nouny>)(?<![溫氏]</ENTITY_oov>)<ENTITY_measurement>[^<]+?[度°]</ENTITY_measurement>|<ENTITY_measurement>[^<]+?[度°]</ENTITY_measurement>(?=<ENTITY_nouny>角<)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.anglePat, indexWithPOS)

        return resultLIST

    def getArea(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「週邊地區」的字串
        '''
        if self.areaPat !=None:
            pass
        else:
            self.areaPat = re.compile("((<LOCATION>[^<]+?</LOCATION>)|(<KNOWLEDGE_addTW>[^<]+?</KNOWLEDGE_addTW>)|(<KNOWLEDGE_routeTW>[^<]+?</KNOWLEDGE_routeTW>)|(<KNOWLEDGE_place>[^<]+?</KNOWLEDGE_place>)|(<ENTITY_nounHead>[^<]+?</ENTITY_nounHead>)|(<ENTITY_nouny>[^<]+?</ENTITY_nouny>)|(<ENTITY_noun>[^<]+?</ENTITY_noun>)|(<ENTITY_oov>[^<]+?</ENTITY_oov>))(<RANGE_locality>[^<]+?</RANGE_locality>)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.areaPat, indexWithPOS)

        return resultLIST

    def getCapacity(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「容量」的字串
        '''
        if self.capacityPat !=None:
            pass
        else:
            self.capacityPat = re.compile("<ENTITY_measurement>[^<度°]+?([升勺合斗石GMBTcb]|加侖|品脫|(b[iy]tes?)|[Mm][Ll]|mol||oz)</ENTITY_measurement>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.capacityPat, indexWithPOS)

        return resultLIST

    def getDate(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「日期」的字串
        '''
        if self.datePat !=None:
            pass
        else:
            self.datePat = re.compile("(<TIME_year>[^<]+?</TIME_year>)?<TIME_month>[^<]+?</TIME_month><TIME_day>[^<]+?</TIME_day>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.datePat, indexWithPOS)

        return resultLIST

    def getDecimal(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「小數」的字串
        '''
        if self.decimalPat !=None:
            pass
        else:
            self.decimalPat = re.compile("<ENTITY_num>[^<.．點]*?[.．點][^<.．點]+?</ENTITY_num>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.decimalPat, indexWithPOS)

        return resultLIST

    def getDuration(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「時間區間」的字串
        '''
        if self.durationPat !=None:
            pass
        else:
            self.durationPat = re.compile("""(<TIME_justtime>[^<]+?</TIME_justtime>|<TIME_[^>]{3,6}>[^<]+?</TIME_[^>]{3,6}>)(<MODIFIER>一?直</MODIFIER>)?(<ACTION_verb>到</ACTION_verb>|<AUX>到</AUX>|<FUNC_inner>至</FUNC_inner>)(<TIME_justtime>[^<]+?</TIME_justtime>|<TIME_[^>]{3,6}>[^<]+?</TIME_[^>]{3,6}>)|
                                             (<TIME_justtime>[^<]+?分鐘</TIME_justtime>)|(<TIME_day>[^<星禮今明昨前後]+?天</TIME_day>)|(<TIME_week>[^<周週]+?([周週]|個星期|個禮拜)</TIME_week>)|(<TIME_month>[^<周週]+?個月</TIME_week>)|(<TIME_season>[^<]+?</TIME_season>)|(<TIME_year>[^<]+?</TIME_year>)|(<TIME_decade>[^<]+?</TIME_decade>)""", re.X)

        resultLIST = self._getMSRA(ArticutResultDICT, self.durationPat, indexWithPOS)

        return resultLIST

    def getFood(self, ArticutResultDICT, withLocation=False, indexWithPOS=True):
        resultLIST = []
        if "result_pos" in ArticutResultDICT:
            resultLIST = self._getFood(ArticutResultDICT, withLocation=withLocation, indexWithPOS=indexWithPOS)
        elif type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            resultLIST = [self._getFood(x, withLocation=withLocation, indexWithPOS=indexWithPOS) for x in ArticutResultLIST]
        else:
            return None

        return resultLIST

    def getFraction(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「分數」的字串
        '''
        if self.fractionPat !=None:
            pass
        else:
            self.fractionPat = re.compile("<ENTITY_measurement>[^<度°]+?(分之)[^<]+?</ENTITY_measurement>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.fractionPat, indexWithPOS)

        return resultLIST

    def getFrequency(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「頻率」的字串
        '''
        if self.frequencyPat !=None:
            pass
        else:
            self.frequencyPat = re.compile("(<ENTITY_measurement>[^<度°]+?((?<!馬)赫|[Hh]z)</ENTITY_measurement>)|((<QUANTIFIER>每</QUANTIFIER>)?(<TIME_justtime>[^<]+?</TIME_justtime>|<TIME_[^>]{3,6}>[^<]+?</TIME_[^>]{3,6}>))(<ENTITY_classifier>[^<]+?</ENTITY_classifier>|<ACTION_eventQuantifier>[^<]+?</ACTION_eventQuantifier>|<ENTITY_num>[^<]+?</ENTITY_num><ENTITY_nounHead>班</ENTITY_nounHead>)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.frequencyPat, indexWithPOS)

        return resultLIST

    def getInteger(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「整數」的字串
        '''
        if self.integerPat !=None:
            pass
        else:
            self.integerPat = re.compile("<ENTITY_num>[^<.．點]+?</ENTITY_num>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.integerPat, indexWithPOS)

        return resultLIST

    def getLength(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「長度」的字串
        '''
        if self.lengthPat !=None:
            pass
        else:
            self.lengthPat = re.compile("<ENTITY_measurement>[^<]+?(?<!方)(公分|光年|inch|[哩里碼吋呎尺米釐mM])</ENTITY_measurement>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.lengthPat, indexWithPOS)

        return resultLIST

    def getLocation(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「重量」的字串。
        此功能和 ArticutAPI 中的 getLoctionStemLIST() 等效。
        '''

        if self.locationPat != None:
            pass
        else:
            self.locationPat = re.compile("(?<=<LOCATION>)[^<]+?(?=</LOCATION>)|(?<=<KNOWLEDGE_addTW>)[^<]+?(?=</KNOWLEDGE_addTW>)|(?<=<KNOWLEDGE_routeTW>)[^<]+?(?=</KNOWLEDGE_routeTW>)")

        resultLIST = []
        resultAppend = resultLIST.append

        if "result_pos" in ArticutResultDICT:
            for p in ArticutResultDICT["result_pos"]:
                if len(p) > 1:
                    resultAppend([(l.start(), l.end(), l.group(0)) for l in self.locationPat.finditer(p)])
                else:
                    resultAppend([])
            if not indexWithPOS:
                resultLIST = self._segIndexConverterNAIVE(ArticutResultDICT, resultLIST)
        elif type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                resultAppend([])
                for p in x["result_pos"]:
                    if len(p) > 1:
                        resultLIST[i].append([[f.start(), f.end(), self.stripPat.sub("", f.group(0))] for f in self.locationPat.finditer(p)])
                    else:
                        resultLIST[i].append([])
                if not indexWithPOS:
                    resultLIST[i] = self._segIndexConverterNAIVE(x, resultLIST[i])
        else:
            return None

        return resultLIST

    def getMeasure(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中為「測量值」的字串
        '''
        if self.measurePat !=None:
            pass
        else:
            self.measurePat = re.compile("<ENTITY_measurement>[^<]+?</ENTITY_measurement>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.measurePat, indexWithPOS)

        return resultLIST

    def getMoney(self, ArticutResultDICT, greedyBOOL=False, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「金額」的字串
        此功能和 ArticutAPI 中的 getCurrencyLIST() 等效。
        '''
        currencyLIST = []

        if "result_pos" in ArticutResultDICT:
            currencyLIST = self._getMoney(ArticutResultDICT, greedyBOOL=greedyBOOL, indexWithPOS=indexWithPOS)
        elif type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            currencyLIST = [self._getMoney(x, greedyBOOL=greedyBOOL, indexWithPOS=indexWithPOS) for x in ArticutResultLIST]
        else:
            return None

        return currencyLIST

    def getOrdinal(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「序數」的字串
        '''
        if self.ordinalPat !=None:
            pass
        else:
            self.ordinalPat = re.compile("<ENTITY_DetPhrase>第[^<]+?</ENTITY_DetPhrase>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.ordinalPat, indexWithPOS)

        return resultLIST

    def getPercent(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「百分比/千分比/萬分比」的字串
        '''
        if self.percentPat !=None:
            pass
        else:
            self.percentPat = re.compile("<ENTITY_measurement>[百千萬億兆]分之[^<]+?</ENTITY_measurement>|<ENTITY_measurement>[^<]+?[％%‰‱](左右|上下|[多餘])?</ENTITY_measurement>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.percentPat, indexWithPOS)

        return resultLIST

    def getPerson(self, ArticutResultDICT, includePronounBOOL=True, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「人名」的字串
        此功能和 ArticutAPI 中的 getPersonLIST() 等效。
        取出斷詞結果中的人名 (Person)
        若 includePronounBOOL 為 True，則連代名詞 (Pronoun) 一併回傳；若為 False，則只回傳人名。
        '''

        if self.personPat != None:
            pass
        else:
            self.personPat = re.compile("(?<=<ENTITY_person>)[^<]*?(?=</ENTITY_person>)")

        person_pronounPat = self.personPat

        if includePronounBOOL:
            person_pronounPat = re.compile(self.personPat.pattern+"|(?<=<ENTITY_pronoun>)[^<]*?(?=</ENTITY_pronoun>)")

        person_pronounLIST = []
        person_pronounAppend = person_pronounLIST.append

        if "result_pos" in ArticutResultDICT:
            for p in ArticutResultDICT["result_pos"]:
                if len(p)>1:
                    personLIST = [[pn.start(), pn.end(), pn.group(0)] for pn in person_pronounPat.finditer(p)]
                    person_pronounAppend(personLIST)
                else:
                    person_pronounAppend([])
            if not indexWithPOS:
                person_pronounLIST = self._segIndexConverterNAIVE(ArticutResultDICT, person_pronounLIST)

        elif type(ArticutResultDICT) is list:
            ArticutResultLIST = self.mergeBulkResult(ArticutResultDICT)
            for i, x in enumerate(ArticutResultLIST):
                person_pronounAppend([])
                for p in x["result_pos"]:
                    if len(p)>1:
                        personLIST = [[pn.start(), pn.end(), pn.group(0)] for pn in person_pronounPat.finditer(p)]
                        person_pronounLIST[i].append(personLIST)
                    else:
                        person_pronounLIST[i].append([])
                if not indexWithPOS:
                    person_pronounLIST[i] = self._segIndexConverterNAIVE(x, person_pronounLIST[i])
        else:
            return None

        return person_pronounLIST

    def getRate(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「比例」的字串
        '''
        if self.ratePat !=None:
            pass
        else:
            self.ratePat = re.compile("(<ENTITY_measurement>[^<]+?倍</ENTITY_measurement>)|(<ENTITY_num>[^<]+?</ENTITY_num><ACTION_verb>比</ACTION_verb><ENTITY_num>[^<]+?</ENTITY_num>)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.ratePat, indexWithPOS)

        return resultLIST

    def getSpeed(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「速度」的字串
        '''
        if self.speedPat !=None:
            pass
        else:
            self.speedPat = re.compile("(<ENTITY_measurement>[^<]+?馬赫</ENTITY_measurement>)|(<ENTITY_nounHead>[時秒分]速</ENTITY_nounHead>|<ENTITY_nouny>[時秒分]速</ENTITY_nouny>|<ENTITY_noun>[時秒分]速</ENTITY_noun>|<ENTITY_oov>[時秒分]速</ENTITY_oov>)<ENTITY_measurement>[^<]+?(?<!方)(公分|光年|[哩里碼吋呎尺米mM])</ENTITY_measurement>|((((<QUANTIFIER>每</QUANTIFIER>)|(<MODIFIER>每秒</MODIFIER>)|(<ENTITY_num>[一1１]</ENTITY_num>))(<TIME_justtime>[^<]+?</TIME_justtime>|<TIME_[^>]{3,6}>[^<]+?</TIME_[^>]{3,6}>))<ENTITY_measurement>[^<]+?(?<!方)(公分|光年|[哩里碼吋呎尺米mM])</ENTITY_measurement>)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.speedPat, indexWithPOS)

        return resultLIST

    def getTemperature(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「溫度」的字串
        '''
        if self.temperaturePat !=None:
            pass
        else:
            self.temperaturePat = re.compile("((?<![^燒達到]</ACTION_verb>)|(?<=[溫氏]</ENTITY_nounHead>)|(?<=[溫氏]</ENTITY_nouny>)|(?<=[溫氏]</ENTITY_oov>))<ENTITY_measurement>[^<]+?[度℃℉]</ENTITY_measurement>")

        resultLIST = self._getMSRA(ArticutResultDICT, self.temperaturePat, indexWithPOS)

        return resultLIST

    def getTime(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「時間」的字串
        '''
        if self.timePat !=None:
            pass
        else:
            self.timePat = re.compile("(<TIME_justtime>[^<]+?</TIME_justtime>|<TIME_[^>]{3,6}>[^<]+?</TIME_[^>]{3,6}>)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.timePat, indexWithPOS)

        return resultLIST

    def getWeight(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「重量」的字串
        '''
        if self.weightPat !=None:
            pass
        else:
            self.weightPat = re.compile("(<ENTITY_measurement>[^<]+?([克斤噸頓磅兩錢]|[Kk]g|KG|盎斯)</ENTITY_measurement>)")

        resultLIST = self._getMSRA(ArticutResultDICT, self.weightPat, indexWithPOS)

        return resultLIST

    def getWWW(self, ArticutResultDICT, indexWithPOS=True):
        '''
        依 MSRA (微軟亞洲研究院, Microsoft Research Lab Asia) NER 標準取出文本中的描述「URL 連結」的字串
        '''
        if self.wwwPat !=None:
            pass
        else:
            self.wwwPat = re.compile("<KNOWLEDGE_url>.+?(</KNOWLEDGE_url>){1}")

        resultLIST = self._getMSRA(ArticutResultDICT, self.wwwPat, indexWithPOS)

        return resultLIST


if __name__ == "__main__":
    from pprint import pprint

    gNER = GenericNER()

    testLIST = [{"result_list": [{"result_pos": ["<TIME_day>今晚</TIME_day><ACTION_verb>來</ACTION_verb><ACTION_verb>點</ACTION_verb><ENTITY_classifier>一道</ENTITY_classifier><ENTITY_nouny>法式</ENTITY_nouny><ACTION_verb>焗烤</ACTION_verb><ENTITY_nouny>龍蝦</ENTITY_nouny>"]},
                                 {"result_pos": ["<ENTITY_noun>藥燉</ENTITY_noun><ENTITY_noun>排骨</ENTITY_noun><ACTION_verb>加</ACTION_verb><ENTITY_noun>藥燉</ENTITY_noun><ENTITY_noun>土虱</ENTITY_noun>", "，", "<ENTITY_nouny>花生卷</ENTITY_nouny><ACTION_verb>加</ACTION_verb><ENTITY_noun>冰淇淋</ENTITY_noun>"]},
                                 {"result_pos": ['<TIME_year>今年</TIME_year><ENTITY_num>十</ENTITY_num><ENTITY_noun>歲</ENTITY_noun><FUNC_inner>的</FUNC_inner><ENTITY_person>彼德</ENTITY_person><ACTION_verb>有</ACTION_verb><ENTITY_classifier>一個</ENTITY_classifier><ENTITY_num>八</ENTITY_num><ENTITY_noun>歲</ENTITY_noun><FUNC_inner>的</FUNC_inner><ENTITY_pronoun>弟弟</ENTITY_pronoun><FUNC_conjunction>和</FUNC_conjunction><ENTITY_classifier>一個</ENTITY_classifier><ENTITY_num>十四</ENTITY_num><ENTITY_noun>歲</ENTITY_noun><FUNC_inner>的</FUNC_inner><ENTITY_pronoun>姐姐</ENTITY_pronoun>',
                                                 '，',
                                                 '<ENTITY_nouny>測試人</ENTITY_nouny><ENTITY_nounHead>名</ENTITY_nounHead><ENTITY_person>蔡英文</ENTITY_person><FUNC_conjunction>與</FUNC_conjunction><ENTITY_nouny>陳</ENTITY_nouny><FUNC_inner>時</FUNC_inner><LOCATION>中共</LOCATION><MODIFIER>同</MODIFIER><ACTION_verb>參與</ACTION_verb><ENTITY_nouny>會議</ENTITY_nouny>']},
                                 {'result_pos': ['<ENTITY_oov>容量</ENTITY_oov><ENTITY_measurement>1公升</ENTITY_measurement><FUNC_inner>的</FUNC_inner><ENTITY_nouny>牛奶紙盒</ENTITY_nouny><RANGE_locality>裡</RANGE_locality><MODIFIER>只</MODIFIER><MODAL>能</MODAL><ACTION_verb>裝</ACTION_verb>',
                                                 ' ',
                                                 '<ENTITY_measurement>995.5毫升</ENTITY_measurement><FUNC_inner>的</FUNC_inner><ENTITY_nouny>水</ENTITY_nouny>']}],
                 "status": True}]
    resultLIST = gNER.getFood(testLIST, indexWithPOS=False)
    pprint(resultLIST)

    #foodTestDICT = {"result_pos":["<ENTITY_noun>藥燉</ENTITY_noun><ENTITY_noun>排骨</ENTITY_noun><ACTION_verb>加</ACTION_verb><ENTITY_noun>藥燉</ENTITY_noun><ENTITY_noun>土虱</ENTITY_noun>", "，", "<ENTITY_nouny>花生卷</ENTITY_nouny><ACTION_verb>加</ACTION_verb><ENTITY_noun>冰淇淋</ENTITY_noun>"]}
    #foodLIST = gNER.getFood(foodTestDICT, withLocation=True)
    #print(foodLIST)

    #foodTestDICT = {"result_pos": ["<TIME_day>今晚</TIME_day><ACTION_verb>來</ACTION_verb><ACTION_verb>點</ACTION_verb><ENTITY_classifier>一道</ENTITY_classifier><ENTITY_nouny>法式</ENTITY_nouny><ACTION_verb>焗烤</ACTION_verb><ENTITY_nouny>龍蝦</ENTITY_nouny>"]}
    #foodLIST = gNER.getFood(foodTestDICT, withLocation=True)
    #print(foodLIST)



    #ageTestDICT = {"result_pos":["<ENTITY_num>六</ENTITY_num><ENTITY_noun>歲</ENTITY_noun>"]}
    #ageLIST = gNER.getAge(ageTestDICT, False)
    #print(ageLIST)


#!/usr/bin/env python
# -*- coding:utf-8 -*-

from graphene.types.resolver import dict_resolver
import graphene
import sys
import os
import json
import re

from pprint import pprint

# Articut Result 檔案路徑
resultFilePath = ''


"""
Articut GraphQL Schema
"""
class Persons(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    pos_ = graphene.String()
    tag_ = graphene.String()

class Nouns(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    pos_ = graphene.String()
    tag_ = graphene.String()

class Numbers(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    pos_ = graphene.String()
    tag_ = graphene.String()

class Sites(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    pos_ = graphene.String()
    tag_ = graphene.String()

class Userdefined(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    pos_ = graphene.String()
    tag_ = graphene.String()

class Meta(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    lang = graphene.String()
    description = graphene.String()

class Tokens(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    pos_ = graphene.String()
    tag_ = graphene.String()
    isStop = graphene.Boolean()
    isEntity = graphene.Boolean()
    isVerb = graphene.Boolean()
    isTime = graphene.Boolean()
    isClause = graphene.Boolean()
    isKnowledge = graphene.Boolean()

class Ents(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    persons = graphene.List(Persons)
    nouns = graphene.List(Nouns)
    numbers = graphene.List(Numbers)
    sites = graphene.List(Sites)
    #userdefined = graphene.List(Userdefined)

class Doc(graphene.ObjectType):
    class Meta:
        default_resolver = dict_resolver
    
    text = graphene.String()
    tokens = graphene.List(Tokens)
    ents = graphene.Field(Ents)

class Nlp(graphene.ObjectType):
    meta = graphene.Field(Meta)
    doc = graphene.Field(Doc)

class Query(graphene.ObjectType):
    nlp = graphene.Field(
        Nlp,
        filepath = graphene.String(),
        model = graphene.String()
    )
    
    def resolve_nlp(self, info, filepath, model):
        if model != "TW":
            return Nlp(
                meta = {
                    "lang": model,
                    "description": 'Articut-GraphQL Model Unsupported.'
                }
            )
        
        if filepath[-5:] == '.json':
            try:
                with open(filepath, 'r', encoding='utf-8') as resultFile:
                    result = json.loads(resultFile.read())
                textTagLIST = posList2TextTag(result["result_pos"])
                
                return Nlp(
                    meta = {
                        "lang": model,
                        "description": 'Articut GraphQL Query Result.'
                    },
                    doc = {
                        "text": result["result_segmentation"].replace('/', ''),
                        "tokens": getTokens(textTagLIST),
                        "ents": getEnts(textTagLIST)
                    }
                )
            except Exception as e:
                print('[Articut-GraphQL ERROR] {}'.format(e))
        return Nlp(
            meta = {
                "lang": model,
                "description": 'Articut-GraphQL Error.'
            }
        )


"""
Used by ArticutAPI.py
"""
class GraphQL():
    def query(self, filePath, query="""
    {
      meta {
        lang
        description
      }
      doc {
        text
        tokens {
          text
          pos_
          tag_
          isStop
          isEntity
          isVerb
          isTime
          isClause
          isKnowledge
        }
        ents {
          persons {
            text
            pos_
            tag_
          }
          nouns {
            text
            pos_
            tag_
          }
          numbers {
            text
            pos_
            tag_
          }
          sites {
            text
            pos_
            tag_
          }
        }
      }
    }"""):
        query = """{\n  nlp(filepath: "{{filePath}}", model: "TW") {{query}}\n}""".replace('{{filePath}}', filePath).replace('{{query}}', query)
        result = graphene.Schema(query=Query).execute(query)
        return json.loads(json.dumps({"data": result.data}))

"""
將 result_pos 拆開成 [{"text", "tag_", "pos_"} ...]
"""
def posList2TextTag(posLIST):
    textTagLIST = []
    textPosPat = re.compile("<[^>]*?>.*?</[^>]*?>")
    posPat = re.compile("(?<=>).*?</[^>]*?>")
    posLIST.reverse()
    for pos in posLIST:
        if pos[0] == '<' and pos[-1] == '>':
            textPosLIST = [p.group(0) for p in reversed(list(textPosPat.finditer(pos)))]
            for t in textPosLIST:
                textLIST = [tp.group(0).split("</") for tp in posPat.finditer(t)]
                textTagLIST.append({
                    "text": textLIST[0][0],
                    "tag_": textLIST[0][1][:-1],
                    "pos_": pos2UniversalPOS(textLIST[0][1][:-1])
                })
        else:
            textTagLIST.append({
                "text": pos,
                "tag_": 'PUNCTUATION',
                "pos_": 'PUNCT'
            })
    textTagLIST.reverse()
    return textTagLIST

"""
Articut POS 轉換 Universal Part-of-speech Tags
"""
def pos2UniversalPOS(pos):
    if pos in ['FUNC_inner']:
        return 'ADP'
    if pos in ['FUNC_determiner']:
        return 'DET'
    if pos in ['AUX', 'MODAL']:
        return 'AUX'
    if pos in ['ASPECT', 'FUNC_negation']:
        return 'PART'
    if pos in ['FUNC_inter', 'FUNC_conjunction']:
        return 'CONJ'
    if pos in ['ENTITY_person', 'ENTITY_pronoun']:
        return 'PERSON'
    if pos in ['TIME_justtime', 'RANGE_period']:
        return 'TIME'
    if pos in ['QUANTIFIER', 'ENTITY_measurement']:
        return 'QUANTITY'
    if pos in ['MODIFIER', 'MODIFIER_color', 'FUNC_modifierHead']:
        return 'ADJ'
    if pos in ['LOCATION', 'RANGE_locality', 'KNOWLEDGE_place', 'KNOWLEDGE_addTW', 'KNOWLEDGE_route']:
        return 'LOC'
    if pos in ['VerbP', 'ACTION_verb', 'ACTION_lightVerb', 'ACTION_quantifiedVerb']:
        return 'VERB'
    if pos in ['TIME_day', 'TIME_week', 'TIME_month', 'TIME_season', 'TIME_year', 'TIME_decade', 'TIME_holiday']:
        return 'DATE'
    if pos in ['IDIOM', 'ENTITY_noun', 'ENTITY_nouny', 'ENTITY_oov', 'ENTITY_NP', 'ENTITY_nounHead', 'ENTITY_num', 'ENTITY_classifier', 'ENTITY_possessive']:
        return 'NOUN'

    return 'OTHER' # ['UserDefined', 'CLAUSE_AnotAQ', 'CLAUSE_YesNoQ', 'CLAUSE_WhoQ', 'CLAUSE_WhatQ', 'CLAUSE_WhereQ', 'CLAUSE_WhenQ', 'CLAUSE_HowQ', 'CLAUSE_WhyQ', 'CLAUSE_Particle', 'KNOWLEDGE_url']

"""
Articut-GraphQL Content
"""
def getTokens(textTagLIST):
    resultLIST = []
    for textTag in textTagLIST:
        resultDICT = {
            "text": textTag["text"],
            "tag_": textTag["tag_"],
            "pos_": textTag["pos_"],
            "isStop": posIsStop(textTag["tag_"]),
            "isEntity": posIsEntity(textTag["tag_"]),
            "isVerb": posIsVerb(textTag["tag_"]),
            "isTime": posIsTime(textTag["tag_"]),
            "isClause": posIsClause(textTag["tag_"]),
            "isKnowledge": posIsKnowledge(textTag["tag_"])
        }
        resultLIST.append(resultDICT)
    return resultLIST

def getEnts(textTagLIST):
    resultDICT = {
        "persons": getPersons(textTagLIST),
        "nouns": getNouns(textTagLIST),
        "numbers": getNumbers(textTagLIST),
        "sites": getSites(textTagLIST)
        #"userdefined": getUserdefined(textTagLIST)
    }
    return resultDICT

"""
Articut-GraphQL Function
"""
def posIsStop(pos):
    if pos in ['ACTION_lightVerb', 'FUNC_determiner', 'FUNC_modifierHead', 'FUNC_negation', 'FUNC_conjunction', 'RANGE_locality', 'RANGE_period']:
        return True
    return False

def posIsEntity(pos):
    if pos in ['ENTITY_num', 'ENTITY_classifier', 'ENTITY_measurement', 'ENTITY_person', 'ENTITY_pronoun', 'ENTITY_possessive', 'ENTITY_noun', 'ENTITY_nounHead', 'ENTITY_nouny', 'ENTITY_oov', 'ENTITY_NP']:
        return True
    return False

def posIsVerb(pos):
    if pos in ['ACTION_verb', 'ACTION_quantifiedVerb', 'VerbP']:
        return True
    return False

def posIsTime(pos):
    if pos in ['TIME_justtime', 'TIME_holiday', 'TIME_day', 'TIME_week', 'TIME_month', 'TIME_season', 'TIME_year', 'TIME_decade']:
        return True
    return False

def posIsClause(pos):
    if pos in ['CLAUSE_AnotAQ', 'CLAUSE_YesNoQ', 'CLAUSE_WhoQ', 'CLAUSE_WhatQ', 'CLAUSE_WhereQ', 'CLAUSE_WhenQ', 'CLAUSE_HowQ', 'CLAUSE_WhyQ', 'CLAUSE_Particle']:
        return True
    return False

def posIsKnowledge(pos):
    if pos in ['KNOWLEDGE_addTW', 'KNOWLEDGE_url', 'KNOWLEDGE_place', 'KNOWLEDGE_route', 'LOCATION', 'UserDefined']:
        return True
    return False

def getPersons(textTagLIST):
    resultLIST = []
    for textTag in textTagLIST:
        if textTag["tag_"] in ['ENTITY_person', 'ENTITY_pronoun']:
            resultLIST.append(textTag)
    return resultLIST

def getNouns(textTagLIST):
    resultLIST = []
    for textTag in textTagLIST:
        if textTag["tag_"] in ['ENTITY_noun', 'ENTITY_nounHead', 'ENTITY_nouny', 'ENTITY_oov', 'ENTITY_NP']:
            resultLIST.append(textTag)
    return resultLIST

def getNumbers(textTagLIST):
    resultLIST = []
    for textTag in textTagLIST:
        if textTag["tag_"] in ['ENTITY_num', 'ENTITY_classifier', 'ENTITY_measurement']:
            resultLIST.append(textTag)
    return resultLIST

def getSites(textTagLIST):
    resultLIST = []
    for textTag in textTagLIST:
        if textTag["tag_"] in ['KNOWLEDGE_addTW', 'KNOWLEDGE_place', 'LOCATION', 'KNOWLEDGE_route']:
            resultLIST.append(textTag)
    return resultLIST

# Uncompleted
def getUserdefined(textTagLIST):
    resultLIST = []
    for textTag in textTagLIST:
        if textTag["tag_"] in ['']:
            resultLIST.append(textTag)
    return resultLIST



"""
Used by python ArticutGraphQL.py articutResult.json
Starlette server (http://0.0.0.0:8000)
"""
def serverStart():
    from starlette.applications import Starlette
    from starlette.routing import Router
    from starlette.routing import Route
    import uvicorn
    
    app = Router([Route('/', endpoint=graphQL, methods=['GET', 'POST'])])
    uvicorn.run(app, host='0.0.0.0', port=8000)
    return None

async def graphQL(request):
    from starlette.templating import Jinja2Templates
    from starlette.status import HTTP_400_BAD_REQUEST
    from starlette.responses import PlainTextResponse
    from starlette.responses import JSONResponse
    
    if request.method == 'POST':
        content_type = request.headers.get("Content-Type", "")
        if content_type == 'application/json':
            data = await request.json()
        else:
            return PlainTextResponse('Bad Request!', status_code=HTTP_400_BAD_REQUEST)
        
        try:
            query = data["query"]
            variables = data.get("variables")
        except KeyError:
            return PlainTextResponse('Bad Request!', status_code=HTTP_400_BAD_REQUEST)
        
        result = graphene.Schema(query=Query).execute(query, variables=variables)
        return JSONResponse({"data": result.data})
    else:
        return Jinja2Templates(directory='Toolkit').TemplateResponse('graphQL.html', {
            "request": request,
            "resultFilePath": resultFilePath
        })

if __name__ == '__main__':
    if len(sys.argv) >= 2:
        resultFilePath = sys.argv[1]
        if os.path.isfile(resultFilePath):
            serverStart()
        else:
            print('{} 檔案不存在！'.format(resultFilePath))
    else:
        print('請輸入斷詞結果檔案路徑，例：python ArticutGraphQL.py articutResult.json')